#!/usr/bin/python2

import ujson
import os
import sys

from edge.com import fs
from edge.webserver.sysconf.uiconf.pipeline import PipelineUiconf

VS_UICONF_PATH = '/etc/edge/setupstore/admin/actions/pipeline/0/videosource/0.json'
MG_DGCONF_PATH = '/etc/edge/mediagrabber.0/main.conf'

RAMDISK_DIRPATH = '/var/spool/edge'

CONST_300M = 314572800
CONST_250M = 262144000


def _main():
    terms_info = PipelineUiconf.load_conf()
    # if len(terms_info) != 1:
    #     print "Number of pipelines != 1"
    #     sys.exit(1)

    vs_uiconf = fs.load_json3(VS_UICONF_PATH)
    vs_uiconf_pub = vs_uiconf['rw']  # type: dict

    if vs_uiconf_pub['type'] != 'mv':
        print "Not a MV pipeline"
        sys.exit(1)

    if len(sys.argv) == 2 and sys.argv[1] in ('-h', '--help'):
        print "Usage: \n\tmediastore-cleanup-args.py <image_width> <image_height> <image_fps> <file_count>"
        sys.exit(0)

    mg_dgconf = fs.load_json3(MG_DGCONF_PATH)
    mg_driver = mg_dgconf['Driver']  # type: dict
    mg_params = mg_driver['ParamSet']  # type: dict
    mg_source = mg_params['VideoSource']  # type: dict
    mg_acquisition = mg_source['Acquisition']  # type: dict
    mg_sensor = mg_acquisition['Sensor']  # type: dict
    mg_fileapi = mg_dgconf['FileApi']  # type: dict

    image_w = mg_sensor['Width']
    image_h = mg_sensor['Height']
    image_fps = mg_acquisition['Fps']
    file_count = mg_fileapi['FileCountHigh']

    statvfs = os.statvfs(RAMDISK_DIRPATH)
    ramdisk_size = statvfs.f_frsize * statvfs.f_blocks

    mg_store_size = image_w * image_h * file_count

    du_max_mb = (ramdisk_size - mg_store_size) - CONST_300M
    du_min_mb = du_max_mb - CONST_250M

    du_max_perc = float(du_max_mb) / ramdisk_size
    du_min_perc = float(du_min_mb) / ramdisk_size

    result_jnode = {
        'CleanupMode': 2,
        'Cleanup': {
            'CheckPeriod': image_fps,
            'DiskLoUsage': round(du_min_perc, 2),
            'DiskHiUsage': round(du_max_perc, 2),
            'InodeLoMaxUsage': 0.7,
            'InodeHiMaxUsage': 0.8
        }
    }

    print ujson.dumps(result_jnode, indent=2)


if __name__ == '__main__':
    _main()
