#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_controller import ControllerApi


class _Module(cgi.CgiModule):
    """
    GET   /api/about   /
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule

        :raise CgiError:
        """
        method_to_handlers_map = {
            'GET': [
                cls._about__get
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _about__get(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 0:
            eventlog.log_i('fail: http404')
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            dev_info = ControllerApi.about()
            return cgi.CgiReply(body=dev_info)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-about', 'debug')
    try:
        cgi.run_cgi(lambda arg: _Module.factory(arg))
    finally:
        eventlog.close_log()
