from edge import eventlog
from edge.webserver import cgi
from edge.webserver.origins import alarm


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/origins/alarm   /
    PUT   /api/v2/origins/alarm   /<origin>/<name>
    """
    def __init__(self, handler):
        """
        :type handler: () -> CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._alarm__get,
            ],
            'PUT': [
                cls._alarm__put_origin_name
            ],
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _alarm__get(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 0:
            raise cgi.CgiError404

        request.assert_query_present()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            origin = None
            name = None
            for name_value in request.query_params:
                name, value = name_value.split('=', 1)
                if name == 'name':
                    name = value
                elif name == 'origin':
                    origin = value

            if None in [origin, name]:
                raise cgi.CgiError400

            result = alarm.get_alarm(origin, name)
            rc = cgi.R200_SUCCESS if result else cgi.R404_NOT_FOUND
            return cgi.CgiReply(status=rc)

        return cls(handler)

    @classmethod
    def _alarm__put_origin_name(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            origin = request.path_info[0]
            name = request.path_info[1]
            status = cgi.R200_SUCCESS if alarm.add_alarm(origin, name) else cgi.R404_NOT_FOUND
            return cgi.CgiReply(status=status)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-alarm', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
