#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_mv_camera import MvCameraSearchApi
from edge.webserver.sysconf.api_controller import ControllerApi


class _Module(cgi.CgiModule):
    """
    GET    /api/v2/sys/conf/controller   /<uiref>/cameras
    PUT    /api/v2/sys/conf/controller   /<uiref>/reboot
    POST   /api/v2/sys/conf/controller   /<uiref>/units
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._controller__get_uiref_cameras
            ],
            'PUT': [
                cls._controller__put_uiref_reboot
            ],
            'POST': [
                cls._controller__post_uiref_units
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _controller__get_uiref_cameras(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'cameras':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            cams_info = MvCameraSearchApi.list_cams()
            return cgi.CgiReply(body=cams_info)

        return cls(handler)

    @classmethod
    def _controller__put_uiref_reboot(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'reboot':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ctrl_uiref = request.path_info[0]
            ControllerApi.reboot(ctrl_uiref)
            return cgi.CgiReply()

        return cls(handler)

    @classmethod
    def _controller__post_uiref_units(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'units':
            raise cgi.CgiError404

        request.assert_query_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ctrl_uiref = request.path_info[0]
            factory_conf = request.body_jnode if request.body else dict()
            if factory_conf.get('uuid') is not None:
                eventlog.log_e('fail: bad-args')
                raise cgi.CgiError400

            ext_terms_info = ControllerApi.create_subunit(ctrl_uiref, factory_conf)
            return cgi.CgiReply(body=ext_terms_info)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-controller', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
