#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_iit_ca import IitCaApi


class _Module(cgi.CgiModule):
    """
    GET      /api/v2/sys/conf/iit-ca   /<uiref>
    PUT      /api/v2/sys/conf/iit-ca   /<uiref>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._iit_ca__get_uiref,
                cls._iit_ca__get_uiref_factory_limits,
                cls._iit_ca__get_uiref_factory
            ],
            'PUT': [
                cls._iit_ca__put_uiref
            ],
            'POST': [
                cls._iit_ca__post_units_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _iit_ca__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf_pub = IitCaApi.load_service(request.path_info[0])
            return cgi.CgiReply(body=uiconf_pub)

        return cls(handler)

    @classmethod
    def _iit_ca__get_uiref_factory_limits(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        if request.path_info[1] != 'factory':
            raise cgi.CgiError404

        roles = []
        for name_value in request.query_params:
            name, value = name_value.split('=', 1)
            if name == 'role':
                roles.append(value)

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            reply = [
                {
                    "type": "iit-ca~end-user",
                    "role": "unit"
                }
            ]
            return cgi.CgiReply(body=reply)

        return cls(handler)

    @classmethod
    def _iit_ca__get_uiref_factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 3:
            raise cgi.CgiError404

        if request.path_info[1] != 'factory':
            raise cgi.CgiError404

        if request.path_info[2] != 'iit-ca~end-user':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            factory_limits = IitCaApi.load_service_factory(request.path_info[0])
            return cgi.CgiReply(body=factory_limits)

        return cls(handler)

    @classmethod
    def _iit_ca__put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            new_uiconf = IitCaApi.save_service(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=new_uiconf)

        return cls(handler)

    @classmethod
    def _iit_ca__post_units_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        if request.path_info[1] != 'units':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            reply = IitCaApi.make_end_user(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=reply)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-iit-ca', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
