#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_location import LocationApi


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/location    /<uiref>/rotation
    GET   /api/v2/sys/conf/location    /<uiref>
    PUT   /api/v2/sys/conf/location    /<uiref>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._location__get_uiref,
                cls._location__get_uiref_rotation
            ],
            'PUT': [
                cls._location__put_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _location__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf_pub = LocationApi.load_unit(request.path_info[0])
            return cgi.CgiReply(body=uiconf_pub)

        return cls(handler)

    @classmethod
    def _location__put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf_pub = LocationApi.save_unit(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=uiconf_pub)

        return cls(handler)

    @classmethod
    def _location__get_uiref_rotation(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'rotation':
            raise cgi.CgiError404

        request.assert_query_present()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            cam_rotation = LocationApi.estimate_rotation(request.path_info[0], request.query_params)
            return cgi.CgiReply.create200(cam_rotation, envelope=False)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-location', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
