#!/usr/bin/python2

from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver import cgi


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/nls   /<file>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._nls__get_file
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _nls__get_file(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            nls_path = fs.join_path('/opt/edge/share/setupstore/nls', request.path_info[0])
            nls_jnode = fs.load_json3(nls_path, default=None)
            if nls_jnode is None:
                eventlog.log_d('fail: NotFoundError')
                raise NotFoundError

            return cgi.CgiReply.create200(nls_jnode)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-nls', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
