#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_origins import OriginsApi


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/transit
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._get_origins
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _get_origins(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 0:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf = OriginsApi.load_unit()
            return cgi.CgiReply(body=ext_uiconf)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-origins', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
