#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_parking import ParkingApi


class _Module(cgi.CgiModule):
    """
    GET      /api/v2/sys/conf/parking   /<uiref>
    PUT      /api/v2/sys/conf/parking   /<uiref>
    DELETE   /api/v2/sys/conf/parking   /<uiref>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._parking__get_uiref
            ],
            'PUT': [
                cls._parking__put_uiref
            ],
            'DELETE': [
                cls._parking__delete_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _parking__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf = ParkingApi.load_detector(request.path_info[0])
            return cgi.CgiReply(body=uiconf)

        return cls(handler)

    @classmethod
    def _parking__put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            new_uiconf = ParkingApi.save_detector(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=new_uiconf)

        return cls(handler)

    @classmethod
    def _parking__delete_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            terms_info = ParkingApi.drop_detector(request.path_info[0])
            return cgi.CgiReply(body=terms_info)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-parking', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
