from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_root import RootApi


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/root   /root/info
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._root__get_root_info
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _root__get_root_info(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        if request.path_info[0] != 'root':
            raise cgi.CgiError404

        if request.path_info[1] != 'info':
            raise cgi.CgiError404

        request.assert_body_absent()
        request.assert_body_absent()

        def unpack_unit_refs(query_params):
            """
            :type query_params: list[str]
            :rtype: list[str]
            """
            uimod_refs = []
            for name_value in query_params:
                name, value = name_value.split('=', 1)
                if name == 'ref':
                    uimod_refs.append(value)

            return uimod_refs

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            unit_refs = unpack_unit_refs(request.query_params)
            units_info = RootApi.get_units_info(unit_refs)
            return cgi.CgiReply(body={'info': units_info})

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('root-cgi', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
