#!/usr/bin/python2

import os

from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver import cgi
from edge.webserver.sysconf import ph


class _Module(cgi.CgiModule):
    """
    GET     /api/v2/schemas   /<file/path>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._schemas__get_file
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _schemas__get_file(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf_path = fs.join_path(ph.CTRL_UICONF_DIR, *request.path_info)
            uischema_path = fs.join_path(ph.CTRL_UISCHEMA_DIR, *request.path_info)
            if os.path.islink(uischema_path) and fs.path_exists(os.readlink(uischema_path)):
                pass

            elif fs.path_exists(uiconf_path):
                fs.ln_schema(uischema_path, uiconf_path)

            else:
                eventlog.log_d('fail: NotFoundError ({})'.format(uiconf_path))
                return NotFoundError

            schema_jnode = fs.load_json3(uischema_path, default=None)
            if schema_jnode is None:
                eventlog.log_d('fail: NotFoundError')
                raise NotFoundError

            return cgi.CgiReply(body=schema_jnode)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-schema', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
