#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_spot_speed import SpotSpeedApi


class _Module(cgi.CgiModule):
    """
    GET      /api/v2/sys/conf/spot-speed   /<uiref>
    PUT      /api/v2/sys/conf/spot-speed   /<uiref>
    DELETE   /api/v2/sys/conf/spot-speed   /<uiref>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._spot_speed__get_uiref,
            ],
            'PUT': [
                cls._spot_speed__put_uiref
            ],
            'DELETE': [
                cls._spot_speed__delete_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _spot_speed__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            uiconf = SpotSpeedApi.load_detector(request.path_info[0])
            return cgi.CgiReply(body=uiconf)

        return cls(handler)

    @classmethod
    def _spot_speed__put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            new_uiconf = SpotSpeedApi.save_detector(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=new_uiconf)

        return cls(handler)

    @classmethod
    def _spot_speed__delete_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            terms_info = SpotSpeedApi.drop_detector(request.path_info[0])
            return cgi.CgiReply(body=terms_info)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-spot-speed', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
