#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_terminal import TerminalApi


class _Module(cgi.CgiModule):
    """
    GET      /api/v2/sys/conf/terminal   /<uiref>
    GET      /api/v2/sys/conf/terminal   /<uiref>/factory
    PUT      /api/v2/sys/conf/terminal   /<uiref>
    POST     /api/v2/sys/conf/terminal   /<uiref>/units
    DELETE   /api/v2/sys/conf/terminal   /<uiref>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._terminal__get_uiref,
                cls._terminal__get_factory_uiref
            ],
            'PUT': [
                cls._terminal_put_uiref
            ],
            'POST': [
                cls._terminal__post_uiref_units
            ],
            'DELETE': [
                cls._terminal__delete_uiref
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _terminal__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf = TerminalApi.load_terminal(request.path_info[0])
            return cgi.CgiReply(body=ext_uiconf)

        return cls(handler)

    @classmethod
    def _terminal__get_factory_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        if request.path_info[1] != 'factory':
            raise cgi.CgiError404

        request.assert_body_absent()
        roles = []
        for name_value in request.query_params:
            name, value = name_value.split('=', 1)
            if name == 'role':
                roles.append(value)

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            factory_limits = TerminalApi.get_factory_limits(request.path_info[0], roles)
            return cgi.CgiReply(body=factory_limits)

        return cls(handler)

    @classmethod
    def _terminal_put_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_uiconf = TerminalApi.save_terminal(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=ext_uiconf)

        return cls(handler)

    @classmethod
    def _terminal__post_uiref_units(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'units':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            terms_info = TerminalApi.make_unit(request.path_info[0], request.body_jnode)
            return cgi.CgiReply(body=terms_info)

        return cls(handler)

    @classmethod
    def _terminal__delete_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            ext_terms_info = TerminalApi.drop_terminal(request.path_info[0])
            return cgi.CgiReply(body=ext_terms_info)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-terminal', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
