#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_time import TimeApi


class _Module(cgi.CgiModule):
    """
    GET   /api/v2/sys/conf/time   /<uiref>
    PUT   /api/v2/sys/conf/time   /<uiref>
    PUT   /api/v2/sys/conf/time   /<uiref>/manual
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._time__get_uiref
            ],
            'PUT': [
                cls._time__put_uiref,
                cls._time__put_uiref_manual
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    # noinspection PyUnusedLocal
    @classmethod
    def _time__get_uiref(cls, request, limits=None):
        """
        :type request: cgi.CgiRequest
        :type limits: dict | None
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            unit_ext_jnode = TimeApi.load_unit()
            return cgi.CgiReply(body=unit_ext_jnode)

        return cls(handler)

    # noinspection PyUnusedLocal
    @classmethod
    def _time__put_uiref(cls, request, limits=None):
        """
        :type request: cgi.CgiRequest
        :type limits: dict | None
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            unit_ext_jnode = TimeApi.save_unit(request.body_jnode)
            return cgi.CgiReply(body=unit_ext_jnode)

        return cls(handler)

    # noinspection PyUnusedLocal
    @classmethod
    def _time__put_uiref_manual(cls, request, limits=None):
        """
        :type request: cgi.CgiRequest
        :type limits: dict | None
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        elif request.path_info[1] != 'manual':
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            sync_status = TimeApi.sync_manual(sync_source=request.body_jnode)
            return cgi.CgiReply(body=sync_status)

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-time', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
