#!/usr/bin/python2

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf.api_users import UsersApi


class _Module(cgi.CgiModule):
    """
    GET      /api/v2/sys/conf/users   /<uiref>
    PUT      /api/v2/sys/conf/users   /<uiref>/<login>
    """
    def __init__(self, handler):
        """
        :type handler: () -> cgi.CgiReply
        """
        cgi.CgiModule.__init__(self, handler)

    @classmethod
    def factory(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        method_to_handlers_map = {
            'GET': [
                cls._users__get_uiref,
            ],
            'PUT': [
                cls._users__put_uiref_login
            ]
        }

        handlers = method_to_handlers_map.get(request.method, [])
        for handler in handlers:
            try:
                return handler(request)

            except cgi.CgiError404:
                pass

        eventlog.log_d('fail: http404')
        raise cgi.CgiError404

    @classmethod
    def _users__get_uiref(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 1:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_absent()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            unit_ext_jnode = UsersApi.load_unit(request.path_info[0])
            return cgi.CgiReply(body=unit_ext_jnode)

        return cls(handler)

    @classmethod
    def _users__put_uiref_login(cls, request):
        """
        :type request: cgi.CgiRequest
        :rtype: cgi.CgiModule
        """
        if len(request.path_info) != 2:
            raise cgi.CgiError404

        request.assert_query_absent()
        request.assert_body_present()

        def handler():
            """
            :rtype: cgi.CgiReply
            """
            unit_uiref, user_login = request.path_info
            UsersApi.edit_user(unit_uiref, user_login, request.body_jnode)
            return cgi.CgiReply()

        return cls(handler)


if __name__ == '__main__':
    eventlog.open_log('cgi-users', 'debug')
    try:
        cgi.run_cgi(_Module.factory)
    finally:
        eventlog.close_log()
