#! /usr/bin/python
import sys

from edge import eventlog
from edge.exceptions import *
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.api_wiegand_pcb import WiegandPcbApi


class _ArgSet:
    ACTION_CREATE = 'create'
    ACTION_REMOVE = 'remove'

    _REMOVE_ACTION_REF = '--ref'
    _CREATE_ACTION_ENDPOINT = '--endpoint'

    _ARG_VALUE_SEPARATOR = '='

    def __init__(self, action, remove_ref=None, create_endpoint=None):
        """
        :type action: str
        :type remove_ref: str | None
        :type create_endpoint: str | None
        """
        self.action = action
        self.remove_ref = remove_ref
        self.create_endpoint = create_endpoint

    @staticmethod
    def show_usage():
        print 'wiegand-ctl create %s%s<endpoint>' \
              % (_ArgSet._CREATE_ACTION_ENDPOINT, _ArgSet._ARG_VALUE_SEPARATOR)

        print 'wiegand-ctl remove %s%s<wiegand-ref>' \
              % (_ArgSet._REMOVE_ACTION_REF, _ArgSet._ARG_VALUE_SEPARATOR)

    @staticmethod
    def _deserialize_create(raw_arg_set):
        """
        :type raw_arg_set: list of str
        :rtype _ArgSet
        """
        if len(raw_arg_set) != 1:
            raise ValueError('invalid number of arguments')

        raw_name_value = str(raw_arg_set[0]).split(_ArgSet._ARG_VALUE_SEPARATOR, 1)
        if len(raw_name_value) != 2:
            raise ValueError('invalid argument')

        raw_name, raw_value = raw_name_value
        raw_name = raw_name.strip()
        raw_value = raw_value.strip()

        if raw_name != _ArgSet._CREATE_ACTION_ENDPOINT:
            raise ValueError('invalid argument')

        return _ArgSet(_ArgSet.ACTION_CREATE, create_endpoint=raw_value)

    @staticmethod
    def _deserialize_remove(raw_arg_set):
        """
        :type raw_arg_set: list of str
        :rtype _ArgSet
        """
        if len(raw_arg_set) != 1:
            raise ValueError('invalid number of arguments')

        raw_name_value = str(raw_arg_set[0]).split(_ArgSet._ARG_VALUE_SEPARATOR, 1)
        if len(raw_name_value) != 2:
            raise ValueError('invalid argument')

        raw_name, raw_value = raw_name_value
        raw_name = raw_name.strip()
        raw_value = raw_value.strip()

        if raw_name != _ArgSet._REMOVE_ACTION_REF:
            raise ValueError('invalid argument')

        return _ArgSet(_ArgSet.ACTION_REMOVE, remove_ref=raw_value)

    @staticmethod
    def deserialize(raw_arg_set):
        """
        :type raw_arg_set: list of str
        :rtype _ArgSet
        """
        raw_arg_count = len(raw_arg_set)
        if raw_arg_count < 2:
            raise ValueError('invalid number of arguments')

        raw_action = str(raw_arg_set[1])

        if raw_action == _ArgSet.ACTION_CREATE:
            return _ArgSet._deserialize_create(raw_arg_set[2:])

        elif raw_action == _ArgSet.ACTION_REMOVE:
            return _ArgSet._deserialize_remove(raw_arg_set[2:])

        raise ValueError('invalid command-line')


def _main(raw_arg_set):
    arg_set = None

    try:
        arg_set = _ArgSet.deserialize(raw_arg_set)
    except ValueError:
        eventlog.log_x()
        _ArgSet.show_usage()
        exit(1)

    if arg_set.action == _ArgSet.ACTION_CREATE:
        root_info = ph.RootInfoUiconf.get_info()
        ctrl_info = ph.RootInfoUiconf.find_unit_r(root_info, unit_type=ph.UI_TYPE_CONTROLLER)
        WiegandPcbApi.make_service(ctrl_info['ref'], arg_set.create_endpoint)

    elif arg_set.action == _ArgSet.ACTION_REMOVE:
        WiegandPcbApi.drop_service(arg_set.remove_ref)

    exit(0)


if __name__ == '__main__':
    eventlog.open_log('wiegand-ctl', 'debug')
    # noinspection PyBroadException
    try:
        _main(sys.argv)

    except Exception:
        eventlog.log_x()
        exit(1)

    finally:
        eventlog.close_log()
