class ArgSet:
    MODE_NONE = 0
    MODE_VERSION = 1
    MODE_STARTUP = 2

    _PARAM_ID = '--id'
    _PARAM_LOG_LEVEL = '--log-level'
    _PARAM_VERSION = ['--version', '-v']

    _PARAM_NAME_VALUE_SEPARATOR = '='

    _ID_MIN = 0
    _ID_MAX = 65535

    _LOG_LEVEL_SET = ['none', 'error', 'warn', 'info', 'debug', 'trace']
    _DEF_LOG_LEVEL = 'info'

    def __init__(self, raw_arg_set):
        """
        :type raw_arg_set: list of str
        """
        self.mode = ArgSet.MODE_NONE
        self.module_id = None
        self.log_level = ArgSet._DEF_LOG_LEVEL

        arg_count = len(raw_arg_set)
        if not (arg_count > 1):
            return

        if arg_count == 2 and (raw_arg_set[1] in ArgSet._PARAM_VERSION):
            self.mode = ArgSet.MODE_VERSION
            return

        for arg in raw_arg_set[1:]:
            arg_name_value = str(arg).split(ArgSet._PARAM_NAME_VALUE_SEPARATOR, 1)
            if len(arg_name_value) != 2:
                return

            arg_name, arg_value = arg_name_value
            if arg_name == ArgSet._PARAM_ID:
                try:
                    module_id = int(arg_value)
                    if ArgSet._ID_MIN <= module_id <= ArgSet._ID_MAX:
                        self.module_id = module_id
                except ValueError:
                    return

                continue

            if arg_name == ArgSet._PARAM_LOG_LEVEL:
                if arg_value not in ArgSet._LOG_LEVEL_SET:
                    return

                self.log_level = arg_value
                continue

            return

        self.mode = ArgSet.MODE_STARTUP
