from edge.exceptions import *


class CliRequest:
    def __init__(self, cli_path, cli_args):
        """
        :type cli_path: str
        :type cli_args: list[str]
        """
        self.path = cli_path
        self.args = cli_args

    @classmethod
    def create(cls):
        """
        :rtype: CliRequest
        """
        import sys
        return cls.create2(sys.argv)

    @classmethod
    def create2(cls, argv):
        """
        :type argv: list[str]
        :rtype: CliRequest
        """
        return cls(argv[0], argv[1:])

    @property
    def log_name(self):
        """
        :rtype: str
        """
        from os import path
        return path.splitext(path.basename(self.path))[0]

    @property
    def log_level(self):
        """
        :rtype: str
        """
        return 'debug'

    def assert_args(self, count):
        """
        :type count: int
        :rtype: None
        """
        if len(self.args) != count:
            raise NotFoundError

    def assert_min_args(self, count):
        """
        :type count: int
        :rtype: None
        """
        if len(self.args) < count:
            raise NotFoundError

    def assert_arg(self, index, value):
        """
        :type index: int
        :type value: str
        :rtype: None
        """
        if self.args[index] != value:
            raise NotFoundError


class CliResponse:
    EX_OK = 0         # sysexists.h / successful termination
    EX_USAGE = 64     # sysexists.h / command line usage error
    EX_SOFTWARE = 70  # sysexists.h / internal software error

    def __init__(self, cli_status, cli_stdout='', cli_stderr=''):
        """
        :type cli_status: int
        :type cli_stdout: str
        :type cli_stderr: str
        """
        self._status = cli_status
        self._stdout = cli_stdout
        self._stderr = cli_stderr

    @classmethod
    def done(cls, cli_stdout='', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_OK, cli_stdout, cli_stderr)

    @classmethod
    def fail_external(cls, cli_stdout='', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_USAGE, cli_stdout, cli_stderr)

    @classmethod
    def fail_internal(cls, cli_stdout='', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_SOFTWARE, cli_stdout, cli_stderr)

    def render(self):
        """
        :rtype: int
        """
        import sys
        return self.render2(sys.stdout, sys.stderr)

    def render2(self, stdout_stream, stderr_stream):
        """
        :type stdout_stream: IO[str]
        :type stderr_stream: IO[str]
        :rtype: int
        """
        if self._stdout:
            stdout_stream.write(self._stdout)

        if self._stderr:
            stderr_stream.write(self._stderr)

        return self._status


class CliBoolResponse(CliResponse):
    def __init__(self, cli_status, cli_stdout='', cli_stderr=''):
        """
        :type cli_status: int
        :type cli_stdout: str
        :type cli_stderr: str
        """
        CliResponse.__init__(self, cli_status, cli_stdout, cli_stderr)

    @classmethod
    def done(cls, cli_stdout='DONE\n', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_OK, cli_stdout, cli_stderr)

    @classmethod
    def fail_external(cls, cli_stdout='FAIL\n', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_USAGE, cli_stdout, cli_stderr)

    @classmethod
    def fail_internal(cls, cli_stdout='FAIL\n', cli_stderr=''):
        """
        :type cli_stdout: str
        :type cli_stderr: str
        :rtype: CliResponse
        """
        return cls(cls.EX_SOFTWARE, cli_stdout, cli_stderr)


class ICliEngine:
    def __init__(self):
        pass

    def usage(self):
        """
        :rtype: str
        """
        raise NotImplementedError

    def activate(self, request):
        """
        :type request: CliRequest
        :rtype: CliResponse
        """
        raise NotImplementedError
