import datetime

from edge import eventlog
from edge.types import JnodeError
from edge.image.types import (ImFooterArgs, ImFooterColArgs, ImFooterRowArgs,
                              SceneImAnnoArgs, VehicleImArgs, VehicleImAnnoArgs,
                              SceneImArgs, ImEngineArgs, ContrastScheduleConf)

from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf


class JsonContrastScheduleConf(ContrastScheduleConf):
    def __init__(self, *args, **kwargs):
        ContrastScheduleConf.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(args_jnode):
        """
        :type args_jnode: list
        :rtype: ScheduleArgs
        :raise JnodeError:
        """
        mode = ContrastScheduleConf.MODE_NEVER
        time_from = None
        time_to = None
        if len(args_jnode) == 0:
            mode = ContrastScheduleConf.MODE_NIGHT

        elif len(args_jnode) == 1:
            mode = args_jnode[0]
            if mode not in [ContrastScheduleConf.MODE_ALWAYS, ContrastScheduleConf.MODE_NEVER,
                            ContrastScheduleConf.MODE_NIGHT]:
                eventlog.log_e_param('#/mode')
                raise JnodeError

        elif len(args_jnode) == 2:
            mode = args_jnode[0]
            schedule_jnode = args_jnode[1]

            if mode != ContrastScheduleConf.MODE_TIME:
                eventlog.log_e_param('#/mode')
                raise JnodeError

            # deserialize #/time-from
            time_from_jnode = schedule_jnode.get('time-from')

            if not type(time_from_jnode) in [list, tuple] or len(time_from_jnode) != 2:
                eventlog.log_e_param('#/time-from')
                raise JnodeError

            hh, mm = time_from_jnode
            if not (int == type(hh) and 0 <= hh <= 23):
                eventlog.log_e_param('#/time-from/0')
                raise JnodeError

            if not (int == type(mm) and 0 <= mm <= 59):
                eventlog.log_e_param('#/time-from/1')
                raise JnodeError

            time_from = datetime.time(hh, mm)

            # deserialize #/time-to
            time_to_jnode = schedule_jnode['time-to']
            if not type(time_to_jnode) in [list, tuple] or len(time_to_jnode) != 2:
                eventlog.log_e_param('#/time-to')
                raise JnodeError

            hh, mm = time_to_jnode
            if not (int == type(hh) and 0 <= hh <= 23):
                eventlog.log_e_param('#/time-to/0')
                raise JnodeError

            if not (int == type(mm) and 0 <= mm <= 59):
                eventlog.log_e_param('#/time-to/1')
                raise JnodeError

            time_to = datetime.time(hh, mm)

        else:
            mode = ContrastScheduleConf.MODE_NEVER

        return ContrastScheduleConf(mode, time_from, time_to)


class JsonVehicleImAnnoArgs(VehicleImAnnoArgs):
    def __init__(self, *args, **kwargs):
        VehicleImAnnoArgs.__init__(self, *args, **kwargs)


class JsonVehicleImArgs(VehicleImArgs):
    def __init__(self, *args, **kwargs):
        VehicleImArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(profile_jnode):
        """
        :type profile_jnode: dict
        :rtype: JsonVehicleImArgs
        :raise JnodeError:
        """
        # deserialize #/width
        image_w = profile_jnode['width']
        if not (type(image_w) == int and 320 <= image_w <= 800):
            eventlog.log_e_param('#/width')
            raise JnodeError

        # deserialize #/footer
        footer_jnode = profile_jnode.get('footer')
        if dict != type(footer_jnode):
            eventlog.log_e_param('#/footer')
            raise JnodeError

        footer_args = JsonImFooterArgs.deserialize(footer_jnode)
        return JsonVehicleImArgs(image_w, footer_args, JsonVehicleImAnnoArgs(plate=True))


class JsonSceneImAnnoArgs(SceneImAnnoArgs):
    def __init__(self, *args, **kwargs):
        SceneImAnnoArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(anno_jnode):
        """
        :type anno_jnode: dict
        :rtype: JsonSceneImAnnoArgs
        :raise JnodeError:
        """
        vehicle_jnode = anno_jnode.get('vehicle')
        if not vehicle_jnode:
            return SceneImAnnoArgs()

        track = vehicle_jnode.get('track', False)
        body = vehicle_jnode.get('body', False)
        plate = vehicle_jnode.get('plate', False)

        return JsonSceneImAnnoArgs(track, body, plate)


class JsonSceneImArgs(SceneImArgs):
    def __init__(self, *args, **kwargs):
        SceneImArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(profile_jnode):
        """
        :type profile_jnode: dict
        :rtype: JsonSceneImArgs
        :raise JnodeError:
        """

        # deserialize image-width
        image_w = profile_jnode['width']

        # deserialize footer-args
        footer_jnode = profile_jnode.get('footer')
        if not footer_jnode:
            eventlog.log_e('fail: param #/footer has invalid value')
            raise JnodeError

        footer = JsonImFooterArgs.deserialize(footer_jnode)

        # deserialize annotations
        anno_jnode = profile_jnode.get('annotations')
        anno = JsonSceneImAnnoArgs.deserialize(anno_jnode) if anno_jnode else JsonSceneImAnnoArgs()

        return JsonSceneImArgs(image_w, footer, anno)


class JsonImFooterColArgs(ImFooterColArgs):
    def __init__(self, *args, **kwargs):
        ImFooterColArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(col_jnode):
        """
        :type col_jnode: dict
        :rtype: JsonImFooterColArgs
        :raise JnodeError:
        """
        if dict != type(col_jnode):
            eventlog.log_e_param('#/')
            raise JnodeError

        # deserialize #/width
        width = col_jnode.get('width')
        if not (int == type(width) and 1 <= width <= 65535):
            eventlog.log_e_param('#/width')
            raise JnodeError

        # deserialize #/align
        raw_align = col_jnode.get('align')
        if not type(raw_align) in [unicode, str]:
            eventlog.log_e_param('#/align')
            raise JnodeError

        if raw_align == 'left':
            align = -1
        elif raw_align == 'center':
            align = 0
        elif raw_align == 'right':
            align = +1
        else:
            eventlog.log_e_param('#/align')
            raise JnodeError

        # deserialize #/format
        template = col_jnode.get('format')
        if not (type(raw_align) in [unicode, str] and 0 <= len(template) <= 128):
            eventlog.log_e_param('#/format')
            raise JnodeError

        return JsonImFooterColArgs(width, align, template)


class JsonImFooterRowArgs(ImFooterRowArgs):
    def __init__(self, *args, **kwargs):
        ImFooterRowArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(row_jnode):
        """
        :type row_jnode: dict
        :rtype: JsonImFooterRowArgs
        :raise JnodeError:
        """
        if dict != type(row_jnode):
            eventlog.log_e_param('#/')
            raise JnodeError

        # deserialize #/enabled
        enabled = row_jnode.get('enabled')
        if bool != type(enabled):
            eventlog.log_e_param('#/enabled')
            raise JnodeError

        if enabled:
            cols_jnode = row_jnode.get('cols')
            if list != type(cols_jnode):
                eventlog.log_e_param('#/cols')
                raise JnodeError

            cols = map(JsonImFooterColArgs.deserialize, cols_jnode)

        else:
            cols = []

        return JsonImFooterRowArgs(enabled, cols)


class JsonImFooterArgs(ImFooterArgs):
    def __init__(self, *args, **kwargs):
        ImFooterArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(footer_jnode):
        """
        :type footer_jnode: dict
        :rtype: JsonImFooterArgs
        :raise JnodeError:
        """
        # deserialize #/font-height
        font_size = footer_jnode.get('font-height')
        if not (int == type(font_size) and 8 <= font_size <= 72):
            eventlog.log_e_param('#/font-height')
            raise JnodeError

        # deserialize #/rows
        rows_jnode = footer_jnode.get('rows')
        if not (list == type(rows_jnode) and len(rows_jnode) > 0):
            eventlog.log_e_param('#/rows')
            raise JnodeError

        rows = filter(lambda row: row.enabled, map(JsonImFooterRowArgs.deserialize, rows_jnode))

        if len(rows) > 0:
            height = int((1.4 * font_size) * len(rows) + 0.4 * font_size)  # 120%
        else:
            # no active rows in footer
            height = 0

        return JsonImFooterArgs(height, font_size, rows)


class JsonImEngineArgs(ImEngineArgs):
    def __init__(self, *args, **kwargs):
        ImEngineArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(args_jnode):
        """
        :type args_jnode: dict
        :rtype: JsonImEngineArgs
        :raise JnodeError:
        """

        # deserialize profile "scene.measurement.alt"
        scene_jnode = args_jnode.get('scene')
        if not scene_jnode:
            eventlog.log_e('fail: param #/scene has invalid value')
            raise JnodeError

        scene = JsonSceneImArgs.deserialize(scene_jnode)

        scene_footer_jnode = scene_jnode.get('footer')
        if not scene_footer_jnode:
            raise ValueError('#/images/scene/footer')

        # deserialize profile "vehicle"
        vehicle_jnode = args_jnode.get('vehicle')
        vehicle = body_footer_jnode = None
        if vehicle_jnode:
            vehicle = JsonVehicleImArgs.deserialize(vehicle_jnode)

            body_footer_jnode = vehicle_jnode.get('footer')
            if not body_footer_jnode:
                raise ValueError('#/images/vehicle/footer')

        contrast_jnode = CtrlUiconf.get_contrast_vehicle_im()
        contrast_args = JsonContrastScheduleConf.deserialize(contrast_jnode)

        return JsonImEngineArgs(
            scene=scene,
            scene_footer=scene_footer_jnode,
            vehicle=vehicle,
            vehicle_footer=body_footer_jnode,
            contrast_args=contrast_args)
