from edge import eventlog
from edge.types import JnodeError
from ..types import LandmarkServiceArgs


class JsonLandmarkServiceArgs(LandmarkServiceArgs):
    def __init__(self, *args, **kwargs):
        LandmarkServiceArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(camera_conf_jnode, location_jnode):
        """
        :type camera_conf_jnode: dict
        :type location_jnode: dict
        :rtype: JsonLandmarkServiceArgs | None
        :raises JnodeError:
        """
        intrinsic_jnode = camera_conf_jnode.get('ro', {}).get('intrinsic') or \
                          camera_conf_jnode.get('rw', {}).get('conf', {}).get('intrinsic')

        if intrinsic_jnode is None:
            eventlog.log_e_param('#/ro/intrinsic')
            raise JnodeError

        if not intrinsic_jnode.get('active', True):
            return None

        location_args_jnode = location_jnode.get('rw', {}).get('args')
        if location_args_jnode is None:
            eventlog.log_e_param('#/rw/args')
            raise JnodeError

        camera_location_jnode = location_args_jnode.get('camera')
        if camera_location_jnode is None:
            eventlog.log_e_param('#/camera')
            raise JnodeError

        # deserialize position
        position_jnode = camera_location_jnode.get('position')
        if position_jnode is None:
            eventlog.log_e_param('#/position')
            raise JnodeError

        forward = position_jnode.get('forward')
        left = position_jnode.get('left')
        up = position_jnode.get('up')

        if None in [forward, left, up]:
            eventlog.log_e_param('#/position')
            raise JnodeError

        # deserialize rotation
        rotation_jnode = camera_location_jnode.get('rotation')
        if rotation_jnode is None:
            eventlog.log_e_param('#/rotation')
            raise JnodeError

        elevation = rotation_jnode.get('elevation')
        azimuth = rotation_jnode.get('azimuth')
        roll = rotation_jnode.get('roll')

        if None in [elevation, azimuth, roll]:
            eventlog.log_e_param('#/rotation')
            raise JnodeError

        # deserialize frustrum
        frustum_jnode = intrinsic_jnode.get('frustum')

        if frustum_jnode is None:
            eventlog.log_e_param('#/frustum')
            raise JnodeError

        focal_dist = frustum_jnode.get('focal-dist')
        if focal_dist is None:
            eventlog.log_e_param('#/frustum/focal-dist')
            raise JnodeError

        sensor_jnode = frustum_jnode.get('sensor-wh')
        if sensor_jnode is None or len(sensor_jnode) != 2:
            eventlog.log_e_param('#/frustrum/sensor-wh')
            raise JnodeError

        sensor_w_mm, sensor_h_mm = sensor_jnode

        clip_jnode = frustum_jnode.get('forward', [5, 100])
        if clip_jnode is None or len(clip_jnode) != 2:
            eventlog.log_e_param('#/frustrum/forward')
            raise JnodeError

        clip_near, clip_far = clip_jnode

        # deserialize image-wh
        image_jnode = intrinsic_jnode.get('image-wh')

        if image_jnode is None or len(image_jnode) != 2:
            eventlog.log_e_param('#/image-wh')
            raise JnodeError

        sensor_w_px, sensor_h_px = image_jnode

        return JsonLandmarkServiceArgs(
            left, up, forward, elevation, azimuth, roll, clip_near, clip_far,
            sensor_w_mm, sensor_h_mm, sensor_w_px, sensor_h_px, focal_dist)
