from shapely.geometry import polygon, point


class LaneDetector:
    def __init__(self, lane_ident_to_polygon_map, road_img_width, road_img_height):
        """
        :type lane_ident_to_polygon_map: list of (int, list of (int, int)) | list of (str, list of (int, int))
        :type road_img_width: int
        :type road_img_height: int
        """
        self._lane_ident_to_polygon_map = []

        for lane_id, polymask in lane_ident_to_polygon_map:
            self._lane_ident_to_polygon_map.append((lane_id, polygon.Polygon(polymask)))

        if len(self._lane_ident_to_polygon_map) > 0 and (road_img_height <= 0 or road_img_width <= 0):
            raise ValueError

        self._road_img_width = float(road_img_width)
        self._road_img_height = float(road_img_height)

    def get_lane(self, x, y):
        """
        :param x: x coordinate of point [0, 1.0]
        :type x: float
        :param x: y coordinate of point [0, 1.0]
        :type y: float
        :rtype set of int | set of str
        """
        return self.get_lane3(int(x * self._road_img_width), int(y * self._road_img_height))

    def get_lane2(self, left, top, right, bottom):
        """
        :type left: float
        :type top: float
        :type right: float
        :type bottom: float
        :rtype set of int | set of str
        """

        return LaneDetector.get_lane(self, ((left + right) / 2), ((top + bottom) / 2))

    def get_lane3(self, x, y):
        """
        :param x: x coordinate of point [0, road_img_width] px
        :type x: int
        :param y: y coordinate of point [0, road_img_height] px
        :type y: int
        :rtype set of int | set of str
        """
        lane_set = set()
        if not len(self._lane_ident_to_polygon_map):
            return lane_set

        plate_center = point.Point(int(x), int(y))
        for ident, poly in self._lane_ident_to_polygon_map:
            if poly.contains(plate_center):
                lane_set.add(ident)

        return lane_set
