class Geoposition:
    def __init__(self, latitude, longitude, altitude=None, speed=None, azimuth=None):
        """
        :param (int | float) latitude: Latitude in degrees: +/- signifies North/South
        :param (int | float) longitude: Longitude in degrees: +/- signifies East/West
        :param (float | None) altitude: Altitude in meters
        :param (float | None) speed: Speed over ground, meters per second
        :param (float | None) azimuth:
        """

        # http://catb.org/gpsd/gpsd_json.html
        self.latitude = latitude
        self.longitude = longitude
        self.altitude = altitude
        self.speed = speed
        self.azimuth = azimuth

    def __str__(self):
        """
        :rtype: str
        """
        # <latChar><DDD.uuuuuu> <longChar><ddd.uuuuuu>
        geodata_str = '{}{:.6f} {}{:.6f}'.format(
            'N' if self.latitude >= 0 else 'S', abs(self.latitude),
            'E' if self.longitude >= 0 else 'W', abs(self.longitude))
        return geodata_str

    def __eq__(self, other):
        """
        :type other: Geoposition
        :rtype: bool
        """
        if not isinstance(other, self.__class__):
            return False

        dlat = abs(self.latitude - other.latitude)
        dlong = abs(self.longitude - other.longitude)

        return dlat < 0.01 and dlong < 0.01

    def __ne__(self, other):
        return not self.__eq__(other)


def encode_geoposition(self):
    """
    :type self: Geoposition
    :rtype: dict
    """
    jnode = {
        'latitude': self.latitude,
        'longitude': self.longitude
    }

    if self.azimuth is not None:
        jnode['azimuth'] = self.azimuth

    return jnode


def decode_geoposition(jnode):
    """
    :type jnode: dict
    :rtype: Geoposition
    """
    latitude = jnode.get('latitude', 0.0)
    longitude = jnode.get('longitude', 0.0)
    altitude = jnode.get('altitude')
    speed = jnode.get('speed')
    azimuth = jnode.get('azimuth')

    return Geoposition(latitude, longitude, altitude, speed, azimuth)
