from edge.webserver.sysconf import ph
from edge.model.geoposition import Geoposition


class TzServiceArgs:
    def __init__(self, tz_info):
        """
        :type tz_info: datetime.tzinfo
        """
        self.tz_info = tz_info


class GpsdServiceArgs:
    PROVIDER_STATIC = 'static'
    PROVIDER_GPSD = 'gpsd'
    _MAX_ENTRY_COUNT = 100
    _MAX_AGE_DELTA_USEC = int(60 * 1e6)
    _MAX_LOOKUP_DIST_USEC = int(5 * 1e6)

    def __init__(self, provider=PROVIDER_STATIC, geodata=None, endpoint=None, azimuth=None,
                 max_entry_count=_MAX_ENTRY_COUNT,
                 max_age_delta_usec=_MAX_AGE_DELTA_USEC,
                 max_lookup_dist_usec=_MAX_LOOKUP_DIST_USEC):
        """
        :type provider: str
        :type geodata: Geoposition | None
        :type endpoint: (str, int) | None
        :type azimuth: float | None
        :type max_entry_count: int
        :type max_age_delta_usec: int
        :type max_lookup_dist_usec: int
        """
        self.provider = provider
        self.azimuth = azimuth
        self.geodata = geodata
        self.endpoint = endpoint
        self.max_entry_count = max_entry_count
        self.max_age_delta_usec = max_age_delta_usec
        self.max_lookup_dist_usec = max_lookup_dist_usec


class AboutServiceArgs:
    def __init__(self, origin_about_path, server_about_path=ph.CTRL_UICONF_PATH):
        """
        :type origin_about_path: str
        :type server_about_path: str
        """
        self.origin_about_path = origin_about_path
        self.server_about_path = server_about_path
