from string import Template


class JnodeError(ValueError):
    pass


class BufferSizeError(BufferError):
    def __init__(self, min_size):
        """
        :type min_size: int
        """
        self.min_size = min_size


class ImageRef:
    def __init__(self, ts=None, url=None, codec=None, width=None, height=None):
        """
        :type ts: int | None
        :type url: str | None
        :type codec: str | None
        :type width: int | None
        :type height: int | None
        """
        self.ts = ts
        self.url = url
        self.codec = codec
        self.width = width
        self.height = height


class FormatTemplate(Template):
    idpattern = r'[_a-z][-_a-z0-9]*'

    _KNOWN_PHOLDERS = [
        'datetime',
        'datetime-ms',
        'datetime-control',
        'datetime-control-ms',
        'time-control-ms',
        'post-name',
        'geolocation',
        'speed',
        'plate',
        'serial',
        'lane',
        'dir',
        'video-speed',
        'video-speed2',
        'radar-speed',
        'radar-speed2',
        'max-speed',
        'speed-limit',
        'radar-overspeed',
        'model',
        'vendor',
        'section-speed',
        'section-overspeed',
        'wim-speed',
        'main-speed',
        'control-speed',
        'main-overspeed',
        'wim-overspeed'
    ]

    @staticmethod
    def get_pholders(string_format):
        """
        :type string_format: str
        :rtype: list of str
        :raise: ValueError
        """

        pholder_to_true_map = {}
        template = FormatTemplate(string_format)
        while True:
            try:
                template.substitute(pholder_to_true_map)
                return pholder_to_true_map.keys()

            except KeyError as x:
                if x.message in FormatTemplate._KNOWN_PHOLDERS:
                    pholder_to_true_map[x.message] = True
                else:
                    raise ValueError(x.message)
