from edge import eventlog
from edge.exceptions import *
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.uimod.systest import SystestUimod


# noinspection PyClassHasNoInit
class ControllerApi:
    @classmethod
    def about(cls):
        """
        :rtype: dict
        """
        from socket import gethostname
        from edge.service import about

        ctrl_info = {
            'hostname': gethostname(),
            'sw': about.get_sw_version(about.SW_DIRPATH),
            'sw-name': about.get_sw_name() or 'EDGE',
            'sw-lang': about.get_sw_lang()
        }

        hw_version = about.get_hw_version()
        if hw_version:
            ctrl_info['hw'] = hw_version

        return {
            'device': ctrl_info
        }

    # noinspection PyUnusedLocal
    @classmethod
    def create_subunit(cls, ctrl_uiref, factory_conf):
        """
        :type ctrl_uiref: str
        :type factory_conf: dict
        :rtype: dict
        """
        unit_type = factory_conf.get('type')
        if unit_type == ph.UI_TYPE_EVENTSTORE:
            from edge.webserver.sysconf.api_eventstore import EventstoreApi
            return EventstoreApi.make_unit(factory_conf)

        elif unit_type != ph.UI_TYPE_TERMINAL:
            eventlog.log_d('fail: ParamsError')
            raise ParamsError

        elif len(factory_conf) == 1:
            from edge.webserver.sysconf.api_terminal import TerminalApi
            return TerminalApi.create_rtsp_term(ctrl_uiref, factory_conf)

        else:
            from edge.webserver.sysconf.api_terminal import TerminalApi
            return TerminalApi.create_mv_term(ctrl_uiref, factory_conf)

    # noinspection PyUnusedLocal
    @classmethod
    def reboot(cls, ctrl_uiref):
        """
        :type ctrl_uiref: str
        :rtype: None
        """
        import os
        os.system('shutdown -r now > /dev/null 2>&1')
        # os.system('shutdown -r +1 > /dev/null 2>&1')

    @classmethod
    def _set_systest(cls, value):
        """
        :type value: bool
        :rtype: None
        """
        from edge.webserver.sysconf.dgconf.cron import CronDgconf
        from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf
        from edge.webserver.sysconf import edge_creator

        CtrlUiconf.set_systest(value)
        CronDgconf.enable() if value else CronDgconf.disable()

        uimod = ph.UiMod('0', ph.UI_TYPE_CONTROLLER, '0')
        root_info = ph.RootInfoUiconf.get_info()
        uirefs = ph.InfoUiconf.find_unit_refs_r(root_info, unit_types=[
            ph.UI_TYPE_TRAFFIC_RULES, ph.UI_TYPE_FUSION])
        edge_creator.generate(uimod.create_backup_dirpath(), uirefs, forced=True)

        # ph.change_eventstore_systest(True)  # TODO change to generate
        # # TODO: after changing to generate this will happen automatic
        # dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_SPEEDCONTROL)
        # dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_TRACKER)
        dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_EVENTSTORE)

    @classmethod
    def start_systest(cls):
        """
        :rtype: None
        """
        cls._set_systest(True)

    @classmethod
    def finish_systest(cls):
        """
        :rtype: None
        """
        cls._set_systest(False)

    @classmethod
    def install_systest_crt(cls, crt_jnode):
        """
        :type crt_jnode: dict
        :rtype: None
        """
        SystestUimod.install_crt(crt_jnode)
        dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_SPEEDCONTROL)

    @classmethod
    def uninstall_systest_crt(cls):
        """
        :rtype: None
        """
        SystestUimod.uninstall_crt()
        dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_SPEEDCONTROL)

    @classmethod
    def install_systest_report(cls, report_ref, report_path):
        """
        :type report_ref: str
        :type report_path: str
        :rtype: None
        """
        SystestUimod.install_report(report_ref, report_path)

    @classmethod
    def uninstall_systest_report(cls, report_ref):
        """
        :type report_ref: str
        :rtype: None
        """
        SystestUimod.uninstall_report(report_ref)
