
from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.types import FormatTemplate
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.uiconf.terminal import TerminalUiconf

import edge_creator as creator


# noinspection PyClassHasNoInit
class HttpStreamingApi:
    @staticmethod
    def load_unit(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_HTTP_STREAMING)
        cam_dgstats = ph.CamDgStats.create(uimod.pid)
        cam_dgstats.test_availability()
        cam_stats_jnode = cam_dgstats.get_stats(uimod.pid)

        uiconf = uimod.load_conf()
        uiconf_pub = uiconf['rw']
        return {
            'type': ph.UI_TYPE_HTTP_STREAMING,
            'conf': uiconf_pub,
            'stats': cam_stats_jnode
        }

    @classmethod
    def save_unit(cls, uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_HTTP_STREAMING)
        cam_dgstats = ph.CamDgStats.create(uimod.pid)
        cam_dgstats.test_availability()

        def validate_uiconf_pub():
            fs.validate_json(new_uiconf_conf, uimod.schema_path)

            # validate 'format' jnodes from json
            footer_rows = new_uiconf_conf['live']['footer']['rows']
            if cls._validate_format(footer_rows) is False:
                eventlog.log_d('fail: ParamsError')
                raise ParamsError

        validate_uiconf_pub()

        restart_setupstore = []
        uiconf = uimod.load_conf()
        if uiconf['rw'] != new_uiconf_conf:

            if uiconf['rw']['enabled'] != new_uiconf_conf['enabled']:
                restart_setupstore.append(True)

            uiconf['rw'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        if restart_setupstore:
            dg.LifeCycle.sighup_ss_dgmod()

        cam_stats_jnode = cam_dgstats.get_stats(uimod.pid)
        return {
            'type': ph.UI_TYPE_HTTP_STREAMING,
            'conf': new_uiconf_conf,
            'stats': cam_stats_jnode
        }

    @staticmethod
    def _validate_format(jnode):
        """
        :type jnode: list
        :rtype: bool
        """
        key_to_true_map = {
            'datetime': True,
            'datetime-ms': True,
            'post-name': True,
            'geolocation': True,
            'vendor': True,
            'model': True,
            'serial': True
        }

        for row in jnode:
            for col in row['cols']:
                try:
                    FormatTemplate(template=col['format']).substitute(key_to_true_map)
                except KeyError as x:
                    eventlog.log_d('fail: invalid key ({})'.format(x.message))
                    return False

        return True
