from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph
from edge.com.sh import exec_shell
import ujson

import edge_creator as creator


def get_stats_limits(host, port):
    """
    :type host:str
    :type port:int
    :rtype: dict | None
    """
    command_line = 'env RPC_TARGET={}:{} /opt/edge/bin/ipcom-cli get-info'.format(host, port)

    try:
        stdout, exitcode = exec_shell(command_line)
        if exitcode != 0:
            eventlog.log_e('fail: ipcom-cli {} <{}>'.format(exitcode, stdout))
            return None
        return ujson.loads(stdout)

    except InternalError:
        eventlog.log_e('fail: ipcom-cli get-info')
        return None


# noinspection PyClassHasNoInit
class IpcomWagaCameraApi:
    @staticmethod
    def load_unit(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_IPCOM_WAGA)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        ext_uiconf_pub = uiconf['rw']
        server_jnode = ext_uiconf_pub['conf']['server']
        stats_jnode = get_stats_limits(server_jnode['host'], server_jnode['port'])

        reply = {
            'type': ph.UI_TYPE_IPCOM_WAGA,
            'conf': ext_uiconf_pub['conf']
            # "stats": {
            #     "uuid": "pidhorivka-uuid",
            #     "name": "WIM7",
            #     "vendor": "camea",
            #     "version": "0.0.2",
            #     "road": "M-22 Poltava-Oleksandria, km  74+810",
            #     "latitude": 49.1795,
            #     "longitude": 33.7992,
            #     "last-event": [
            #         1929233,
            #         "2021-11-08T15:40:00.075Z"
            #     ]
            # },
            # "limits": {
            #     "lanes": [
            #         {
            #             "name": "PI-KR-W1",
            #             "desc": "PP: Poltava - Oleksandria"
            #         },
            #         {
            #             "name": "PI-PL-W1",
            #             "desc": "ZP: Oleksandria - Polatava"
            #         }
            #     ]
            # }
        }
        if stats_jnode:
            reply.update(stats_jnode)

        return reply

    @staticmethod
    def save_unit(uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_IPCOM_WAGA)

        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        fs.validate_json(new_uiconf, uimod.schema_path)
        server_jnode = new_uiconf_conf['server']
        stats_jnode = get_stats_limits(server_jnode['host'], server_jnode['port'])

        uiconf = uimod.load_conf()
        if uiconf['rw']['conf'] != new_uiconf_conf:
            uiconf['rw']['conf'] = new_uiconf_conf
            if stats_jnode:
                uiconf['ro']['vendor'] = stats_jnode['stats']['vendor']

            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})


        reply = {
            'type': ph.UI_TYPE_IPCOM_WAGA,
            'conf': new_uiconf_conf,
            # "stats": {
            #     "uuid": "pidhorivka-uuid",
            #     "name": "WIM7",
            #     "vendor": "camea",
            #     "version": "0.0.2",
            #     "road": "M-22 Poltava-Oleksandria, km  74+810",
            #     "latitude": 49.1795,
            #     "longitude": 33.7992,
            #     "last-event": [
            #         1929233,
            #         "2021-11-08T15:40:00.075Z"
            #     ]
            # },
            # "limits": {
            #     "lanes": [
            #         {
            #             "name": "PI-KR-W1",
            #             "desc": "PP: Poltava - Oleksandria"
            #         },
            #         {
            #             "name": "PI-PL-W1",
            #             "desc": "ZP: Oleksandria - Polatava"
            #         }
            #     ]
            # }
        }

        if stats_jnode:
            reply.update(stats_jnode)

        return reply
