from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph
from edge.com import db


# noinspection PyClassHasNoInit
class OriginsApi:
    @staticmethod
    def load_unit():
        """
        :rtype: dict
        """
        origins = db.get_origins() or list()

        for origin_jnode in origins:
            uuid = origin_jnode['uuid']
            term_pid = ph.RootInfoUiconf.get_term_pid(uuid)
            origin_jnode['services'] = {
                'event': ['feed', 'arch', 'stats']
            }

            if term_pid is None:
                continue

            origin_jnode['services']['media'] = ['live']

            term_stats_jnode = ph.get_terminal_stats(term_pid)
            origin_jnode['stats'] = term_stats_jnode

            sc_path = '/var/spool/edge/pipeline-{}/zabbix/sc0'.format(term_pid)
            sc_stats = fs.load_json3(sc_path, default=None)
            if sc_stats is None:
                continue

            for unit_jnode in sc_stats['units']:
                if unit_jnode['type'] != 'geolocation':
                    continue

                last_geo = unit_jnode.get('stat', dict()).get('last-geoposition')
                if last_geo is None:
                    continue

                origin_jnode['geolocation'] = {
                    'latitude': last_geo.get('latitude'),
                    'longitude': last_geo.get('longitude'),
                    'sunset': last_geo.get('sunset'),
                    'sunrise': last_geo.get('sunrise')
                }

        reply = {
            'origins': origins
        }

        return reply
