from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph

import edge.webserver.sysconf.edge_creator as creator


# noinspection PyClassHasNoInit
class RoadwayApi:
    @staticmethod
    def load_unit(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_ROADWAY)
        cam_dg_stats = ph.CamDgStats.create(uimod.pid)
        cam_dg_stats.test_availability()
        cam_stats_jnode = cam_dg_stats.get_stats(uimod.pid)

        uiconf = uimod.load_conf()
        ext_uiconf_pub = uiconf['rw']

        reply = {
            'type': ph.UI_TYPE_ROADWAY,
            'conf': ext_uiconf_pub['args'],
            'stats': cam_stats_jnode
        }
        return reply

    @staticmethod
    def save_unit(uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_ROADWAY)
        cam_dg_stats = ph.CamDgStats.create(uimod.pid)
        cam_dg_stats.test_availability()
        cam_stats_jnode = cam_dg_stats.get_stats(uimod.pid)

        def validate_uiconf():
            fs.validate_json({'args': new_uiconf_conf}, uimod.schema_path)

            from shapely.geometry import polygon
            try:
                # validate self-intersection of lanes
                entry_lanes = new_uiconf_conf['lanes']
                for lane_jnode in entry_lanes:
                    polymask = polygon.Polygon(lane_jnode['mask'])
                    if not polymask.is_valid:
                        eventlog.log_d('fail: ParamsError (invalid road-mask)')
                        raise ParamsError

            except (ValueError, IndexError):
                eventlog.log_d('fail: ParamsError (road-mask contains less then 3 vertexes)')
                raise ParamsError

        validate_uiconf()

        uiconf = uimod.load_conf()
        uiconf_pub = uiconf['rw']
        if uiconf_pub['args'] != new_uiconf_conf:
            uiconf_pub['args'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        reply = {
            'type': ph.UI_TYPE_ROADWAY,
            'conf': new_uiconf_conf,
            'stats': cam_stats_jnode
        }
        return reply
