from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph

import edge_creator as creator


# noinspection PyClassHasNoInit
class RtspCameraApi:
    @staticmethod
    def load_unit(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_RTSP_CAMERA)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        ext_uiconf_pub = uiconf['rw']
        stats_jnode = ph.CamDgStats.create(uimod.pid).get_stats(uimod.pid)

        reply = {
            'type': ph.UI_TYPE_RTSP_CAMERA,
            'conf': ext_uiconf_pub['conf'],
            'stats': stats_jnode
        }
        return reply

    @staticmethod
    def save_unit(uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_RTSP_CAMERA)

        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        fs.validate_json(new_uiconf, uimod.schema_path)

        uiconf = uimod.load_conf()
        if uiconf['rw']['conf'] != new_uiconf_conf:
            uiconf['rw']['conf'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        stats_jnode = ph.CamDgStats.create(uimod.pid).get_stats(uimod.pid)
        reply = {
            'type': ph.UI_TYPE_RTSP_CAMERA,
            'conf': new_uiconf_conf,
            'stats': stats_jnode
        }
        return reply
