from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph

import edge.webserver.sysconf.edge_creator as creator


class TcpRecipientApi:
    def __init__(self):
        pass

    @staticmethod
    def make_service(term_pid, factory_conf):
        """
        :type term_pid: str
        :type factory_conf: dict
        :rtype: (str, dict)
        """
        if not ph.RootInfoUiconf.validate_term(term_pid):
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        service_uimod_pid = term_pid
        service_uimod_type = factory_conf.get('type')
        service_name = factory_conf.get('name')

        if service_uimod_type != ph.UI_TYPE_TCP_RECIPIENT or service_name is None:
            eventlog.log_d('fail: ParamsError')
            raise ParamsError

        answer = ph.RootInfoUiconf.add_service(
            service_uimod_pid, service_uimod_type, service_name, ph.UI_TYPE_TERMINAL, [ph.UI_TYPE_LPR])

        if answer is None:
            eventlog.log_e('fail: InternalError')
            raise InternalError

        service_uimod_id, root_info = answer

        uimod = ph.UiMod(service_uimod_pid, service_uimod_type, service_uimod_id)
        uimod.install()
        uiconf = uimod.load_conf()
        uiconf['rw']['conf']['name'] = service_name

        uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
        creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})
        return uimod.ref, root_info

    @staticmethod
    def drop_service(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_TCP_RECIPIENT)
        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        root_info, root_links = ph.RootInfoUiconf.get_info_links()
        _, parent_info = ph.RootInfoUiconf.find_unit_parent_r(root_info, uimod_ref)

        lpr_info = ph.RootInfoUiconf.find_unit_r(parent_info, unit_type=ph.UI_TYPE_LPR)
        lpr_uimod = ph.UiMod.create(lpr_info['ref'])
        lpr_uiconf = lpr_uimod.load_conf(default=True)

        uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref, root_links)
        uirefs.remove(uimod.ref)

        root_info = ph.RootInfoUiconf.del_unit_r(uimod.ref)
        creator.generate(uimod.create_backup_dirpath(), uirefs, {lpr_uimod: lpr_uiconf})
        uimod.uninstall()

        ext_root_info = ph.RootInfoUiconf.get_ext_root_info(root_info)
        reply = {
            'info': [ext_root_info]
        }
        return reply

    @staticmethod
    def load_service(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_TCP_RECIPIENT)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_e('fail: NotFoundError')
            raise NotFoundError

        uiconf_pub = uiconf['rw']

        reply = {
            'type': ph.UI_TYPE_TCP_RECIPIENT,
            'conf': uiconf_pub['conf']
        }
        return reply

    @staticmethod
    def save_service(uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_TCP_RECIPIENT)

        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        def validate_body():
            fs.validate_json({'conf': new_uiconf_conf}, uimod.schema_path)

        validate_body()

        uiconf = uimod.load_conf()
        if uiconf['rw']['conf'] != new_uiconf_conf:
            if new_uiconf_conf['name'] != uiconf['rw']['conf']['name']:
                ph.RootInfoUiconf.set_unit_name(uimod.ref, new_uiconf_conf['name'])

            uiconf['rw']['conf'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        reply = {
            'type': ph.UI_TYPE_TCP_RECIPIENT,
            'conf': new_uiconf_conf
        }
        return reply
