from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver.sysconf import ph
from edge.types import FormatTemplate

import edge.webserver.sysconf.edge_creator as creator


# noinspection PyClassHasNoInit
class TransitApi:
    @staticmethod
    def load_detector(uimod_ref):
        """
        :type uimod_ref: str
        :rtype: dict
        """
        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_TRANSIT)

        uiconf = uimod.load_conf(default=None)
        if uiconf is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        ext_uiconf_pub = uiconf['rw']
        reply = {
            'type': ph.UI_TYPE_TRANSIT,
            'conf': ext_uiconf_pub['args']
        }
        reply.update(ph.get_detector_limits_stats(uimod.pid))
        return reply

    @classmethod
    def save_detector(cls, uimod_ref, new_uiconf):
        """
        :type uimod_ref: str
        :type new_uiconf: dict
        :rtype: dict
        """
        new_uiconf_conf = new_uiconf.get('conf')

        uimod = ph.UiMod.create(uimod_ref)
        uimod.assert_type(ph.UI_TYPE_TRANSIT)

        if uimod.load_conf(default=None) is None:
            eventlog.log_d('fail: NotFoundError')
            raise NotFoundError

        def validate_uiconf_pub():
            fs.validate_json({'args': new_uiconf_conf}, uimod.schema_path)

            images_jnode = new_uiconf_conf['images']
            v_rows = images_jnode['vehicle']['footer']['rows']
            s_rows = images_jnode['scene']['footer']['rows']

            if cls._validate_format(v_rows) is False:
                eventlog.log_d('fail: ParamsError')
                raise ParamsError

            elif cls._validate_format(s_rows) is False:
                eventlog.log_d('fail: ParamsError')
                raise ParamsError

        validate_uiconf_pub()

        uiconf = uimod.load_conf()
        if uiconf['rw']['args'] != new_uiconf_conf:
            if new_uiconf_conf['name'] != uiconf['rw']['args']['name']:
                ph.RootInfoUiconf.set_unit_name(uimod.ref, new_uiconf_conf['name'])

            uiconf['rw']['args'] = new_uiconf_conf
            uirefs = ph.RootInfoUiconf.get_linked_uirefs(uimod.ref)
            creator.generate(uimod.create_backup_dirpath(), uirefs, {uimod: uiconf})

        reply = {
            'type': ph.UI_TYPE_TRANSIT,
            'conf': new_uiconf_conf
        }
        reply.update(ph.get_detector_limits_stats(uimod.pid))
        return reply

    @staticmethod
    def _validate_format(jnode):
        """
        :type jnode: list
        :rtype: bool
        """
        key_to_true_map = {
            'speed': True,
            'datetime': True,
            'datetime-ms': True,
            'datetime-control': True,
            'datetime-control-ms': True,
            'time-control-ms': True,
            'post-name': True,
            'geolocation': True,
            'plate': True,
            'lane': True,
            'dir': True,
            'video-speed': True,
            'video-speed2': True,
            'radar-speed': True,
            'radar-speed2': True,
            'vendor': True,
            'model': True,
            'serial': True,
            'wim-speed': True,
            'main-speed': True,
            'control-speed': True
        }

        for row in jnode:
            for col in row['cols']:
                try:
                    FormatTemplate(template=col['format']).substitute(key_to_true_map)
                except KeyError as x:
                    eventlog.log_d('fail: invalid key (%s)' % x.message)
                    return False

        return True
