import os
import shutil
import subprocess
import time

from edge import eventlog
from edge.com import fs
from edge.exceptions import *
from edge.webserver import cgi
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg

_UISCHEMA_PATH = '/etc/edge/setupstore/schemas/user-password.json'
_UICONF_PATH = '/etc/edge/setupstore/admin/actions/user.json'
_DEF_UICONF_PATH = '/opt/edge/share/setupstore/ui/user-password-jnode.json'
_LOCK_PATH = '/var/run/edge/user-password'
_PASSWD_DGCONF_PATH = '/etc/edge/webserver.0/htpasswd'
_BACKUP_DIRPATH = '/etc/edge/setupstore/backup/user-password.' + \
                  str(round(time.time() * 1000000)) + '/'


def _create_backup():
    # noinspection PyBroadException
    try:
        os.makedirs(_BACKUP_DIRPATH)
        shutil.copy(_PASSWD_DGCONF_PATH, _BACKUP_DIRPATH)
        return True

    except Exception:
        pass

    return False


def _restore_backup():
    # noinspection PyBroadException
    try:
        shutil.copy(_BACKUP_DIRPATH + 'htpasswd', _PASSWD_DGCONF_PATH)
        os.rmdir(_BACKUP_DIRPATH)
        return True

    except Exception:
        return False


def _remove_backup():
    # noinspection PyBroadException
    try:
        shutil.rmtree(_BACKUP_DIRPATH)
    except Exception:
        pass


def _edit_passwd_conf(user_name, user_pass):
    """
    :type user_name: str
    :type user_pass: str
    :rtype: None
    """
    _PROHIBITED_SYMBOLS = ('\\', "'")
    if any(sym in user_pass for sym in _PROHIBITED_SYMBOLS):
        eventlog.log_d('fail: http400 (password contains invalid symbols)')
        raise cgi.CgiError400

    with open(_PASSWD_DGCONF_PATH, 'r') as text_file:
        old_lines = text_file.readlines()
        if not old_lines:
            raise NotImplementedError

    new_lines = []
    for old_line in old_lines:
        if not old_line.startswith(user_name):
            new_lines.append(old_line)

    user_pass = user_pass.replace('"', '\\"')
    openssl_proc = subprocess.Popen(
        'openssl passwd -apr1 "{}"'.format(user_pass),
        shell=True, stdout=subprocess.PIPE)
    new_password = openssl_proc.communicate()[0]
    new_lines.append('{}:{}'.format(user_name, new_password))

    with open(_PASSWD_DGCONF_PATH, 'w') as text_file:
        text_file.write(''.join(new_lines))


# noinspection PyClassHasNoInit
class UsersApi:
    @staticmethod
    def load_unit(unit_uiref):
        """
        :type unit_uiref: str
        :rtype: dict
        """
        def install_uiconf():
            fs.create_jnode_from_default(_DEF_UICONF_PATH, _UICONF_PATH)

        def install_uischema():
            if not os.path.islink(_UISCHEMA_PATH):
                fs.ln_schema(_UISCHEMA_PATH, _UICONF_PATH)

            elif not os.path.exists(os.readlink(_UISCHEMA_PATH)):
                fs.ln_schema(_UISCHEMA_PATH, _UICONF_PATH)

        install_uiconf()
        install_uischema()

        uiconf = fs.load_json3(_UICONF_PATH)
        uiconf_pub = uiconf['rw']  # type: dict
        return {
            'type': ph.UI_TYPE_USERS,
            'conf': uiconf_pub
        }

    @staticmethod
    def edit_user(unit_uiref, user_login, user_uiconf):
        """
        :type unit_uiref: str
        :type user_login: str
        :type user_uiconf: dict
        :rtype: None
        """
        fs.validate_json(user_uiconf, _UISCHEMA_PATH)
        user_pass = user_uiconf['password']

        if not fs.try_lock_pid_file(_LOCK_PATH):
            eventlog.log_e('fail: BusyError')
            raise BusyError

        if not _create_backup():
            eventlog.log_e('fail: InternalError')
            raise InternalError

        # noinspection PyBroadException
        try:
            _edit_passwd_conf(user_login, user_pass)
            dg.LifeCycle.restart_ws_dgmod()

            _remove_backup()

        except Exception:
            eventlog.log_x()
            if _restore_backup():
                _remove_backup()

            raise InternalError
