from edge.com import fs


class BsDgconfEditor65535:
    _DGCONF_PATH = '/etc/edge/blobstore.65535/main.conf'

    def __init__(self, jnode):
        """
        :type jnode: dict
        """
        self._jnode = jnode

    @property
    def jnode(self):
        """
        :rtype: dict
        """
        return self._jnode

    @classmethod
    def load(cls, **kwargs):
        """
        :rtype: BsDgconfEditor65535 | None
        """
        conf_jnode = fs.load_json3(cls._DGCONF_PATH, **kwargs)
        return cls(conf_jnode) if conf_jnode else None

    def save(self):
        """
        :rtype: None
        """
        fs.write_json_file(self.jnode, self._DGCONF_PATH, pretty=True)

    @property
    def chan_count(self):
        """
        :rtype: int
        """
        return len(self.jnode['storage'])

    @property
    def metrics_path(self):
        """
        :rtype: str
        """
        return self.jnode['monitoring']['stat-path']

    def append_chan(self, chanref, es_uiconf):
        """
        :type chanref: str
        :type es_uiconf: dict
        :rtype: None
        """
        cleanup_uiconf = es_uiconf['rw']['args']['cleanup']

        archive_limits = dict()

        if cleanup_uiconf['max-du'][0]:
            archive_limits['max-du'] = cleanup_uiconf['max-du'][1]

        if cleanup_uiconf['min-df'][0]:
            archive_limits['min-df'] = cleanup_uiconf['min-df'][1]

        if cleanup_uiconf['max-abs-time'][0]:
            archive_limits['max-abs-time'] = cleanup_uiconf['max-abs-time'][1] * 60

        if cleanup_uiconf['max-rel-time'][0]:
            archive_limits['max-rel-time'] = cleanup_uiconf['max-rel-time'][1] * 60

        self.jnode['storage'].append({
            'channel': chanref,
            'cache': {
                'directory': '/var/opt/edge/blobstore/65535-cache',
                'limits': {
                    'max-du': 50,
                    'max-abs-time': 1 * 60
                }
            },
            'archive': {
                'directory': '/var/opt/edge/blobstore/65535',
                'filesize': 4 * 1024,
                'limits': archive_limits
            }
        })

    def remove_chan(self, chanref):
        """
        :type chanref: str
        :rtype: None
        """
        self.jnode['storage'] = filter(lambda x: x['channel'] != chanref, self.jnode['storage'])
