from edge.com import fs


# noinspection PyClassHasNoInit
class SsDgconf:
    CONF_PATH = '/etc/edge/setupstore/main.conf'
    CONF_TMP_PATH = CONF_PATH + '.tmp'

    @classmethod
    def load_conf(cls, **kwargs):
        """
        :rtype: dict
        :raises FsError:
        """
        return fs.load_json3(cls.CONF_PATH, **kwargs)

    @classmethod
    def save_conf(cls, conf_jnode):
        """
        :type conf_jnode: dict
        :rtype: None
        """
        fs.write_json_file(conf_jnode, cls.CONF_PATH, pretty=True)

    @classmethod
    def set_bscache_spool_size(cls, size):
        """
        :type size: int
        :rtype: None
        """
        dgconf = cls.load_conf()
        dgconf_groups = dgconf['groups']
        bscache_group = filter(lambda x: x['name'] == 'bs-cache', dgconf_groups)[0]
        bscache_group['spool-size'] = size

        cls.save_conf(dgconf)

    @classmethod
    def append_ss_group(cls, term_pid, dgconf_dirs):
        """
        :type term_pid: str
        :type dgconf_dirs: list[str]
        """
        dgconf = cls.load_conf()

        dgconf['groups'].append({
            'name': 'pipeline-{}'.format(term_pid),
            'spool-size': 740,
            'modules': dgconf_dirs
        })

        cls.save_conf(dgconf)

    @classmethod
    def remove_ss_group(cls, term_pid):
        """
        :type term_pid: str
        """
        dgconf = cls.load_conf()

        group_name = 'pipeline-{}'.format(term_pid)
        dgconf_groups = dgconf['groups']
        group_to_remove = filter(lambda x: x['name'] == group_name, dgconf_groups)
        if len(group_to_remove) == 1:
            dgconf_groups.remove(group_to_remove[0])
            dgconf['groups'] = dgconf_groups
            cls.save_conf(dgconf)
