from edge.com import net
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator
from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf
from edge.webserver.sysconf.uiconf.db import DbUiconf


def _reconfigure_es(endpoints):
    """
    :type endpoints: list[str]
    :rtype: bool
    """
    for _ in [1, 2, 3]:
        try:
            net.query_es_config(endpoints, tout_sec=1)
            return True
        except net.NetError:
            pass

    return False


def _mk_react_path(rule_uimod):
    """
    :type rule_uimod: ph.UiMod
    :rtype: str
    """
    return '/var/spool/edge/system/sc{0}-es65535-fact'.format(rule_uimod.pid)


# noinspection PyUnusedLocal
def es_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if not forced:
        return []

    root_info = ph.RootInfoUiconf.get_info()
    tr_uirefs = ph.RootInfoUiconf.find_unit_refs_r(root_info, ph.UI_TYPE_TRAFFIC_RULES)
    rules_uimods = map(ph.UiMod.create, tr_uirefs)
    return [EsCreator(rules_uimods)]


class EsCreator(BaseCreator):
    def __init__(self, rules_uimods):
        """
        :type rules_uimods: list of ph.UiMod
        """
        BaseCreator.__init__(self)
        self._rules_uimods = rules_uimods
        self._main_conf = None

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return dg.CTRL_ES_DGMOD

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        es_new_conf = {
            "api": net.get_es_uds_endpoint(),
            "db": DbUiconf.CONF_PATH,
            "monitoring": {
                "stat-path": "/var/spool/edge/system/zabbix/es65535",
                "stat-tout": 60
            },
            "fact-react": map(_mk_react_path, self._rules_uimods),
            "systest": CtrlUiconf.get_systest()
        }

        self._main_conf = es_new_conf

    def complete(self):
        es_dgmod = dg.CTRL_ES_DGMOD
        es_dgmod.install()
        es_dgmod.save_main_conf(self._main_conf)
        es_dgmod.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        if not _reconfigure_es(self._main_conf['fact-react']):
            dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_EVENTSTORE)


class EsDeleter:
    def __init__(self, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        """
        self._entry_uimod = entry_uimod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: EsDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_FACTORY:
            return cls(entry_uimod)

    def run(self):
        dgmod = dg.CTRL_ES_DGMOD
        main_conf = dgmod.load_main_conf()

        endpoint = _mk_react_path(self._entry_uimod)
        main_conf['fact-react'] = filter(lambda x: x != endpoint, main_conf['fact-react'])

        dgmod.save_main_conf(main_conf)

        if not _reconfigure_es(main_conf['fact-react']):
            dg.LifeCycle.restart_dgmod(dg.EDGE_TYPE_EVENTSTORE)
