from itertools import ifilter

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator


# noinspection PyUnusedLocal
def lbs_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if entry_uimod.type != ph.UI_TYPE_MEDIASTORE:
        return []

    try:
        ms_uimod = next(ifilter(lambda x: x.type == ph.UI_TYPE_MEDIASTORE, all_used_uimods))
        if forced:
            return [LbsCreator(ms_uimod)]

    except StopIteration:
        pass
    return []


class LbsCreator(BaseCreator):
    def __init__(self, ms_uimod):
        """
        :type ms_uimod: ph.UiMod
        """
        BaseCreator.__init__(self)
        self._ms_uimod = ms_uimod
        self._lbs_dgmod = dg.DgMod.create(dg.EDGE_TYPE_LBSAPP, ms_uimod.pid, ms_uimod.id)
        self._ss = None

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._lbs_dgmod

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        pass

    def complete(self):
        edge_lbs = self._lbs_dgmod
        edge_lbs.install()
        edge_lbs.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class LbsDeleter:
    def __init__(self, dgmod):
        """
        :type dgmod: dg.DgMod
        """
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: LbsDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_MEDIASTORE:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_LBSAPP, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self):
        self._dgmod.uninstall()
