from itertools import ifilter

from copy import deepcopy

from edge.exceptions import *
from edge.com import fs, sh
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator
from edge.webserver.sysconf.uiconf.time import TimeUiconf
from edge.webserver.sysconf.uiconf.db import DbUiconf
from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf
from edge.webserver.sysconf.uimod.systest import SystestUimod


def _mk_eventstore_react_path(ui_pid):
    """
    :type ui_pid: str
    :rtype: str
    """
    return '/var/spool/edge/system/sc{0}-es65535-fact'.format(ui_pid)


# noinspection PyUnusedLocal
def sc_get_creators(uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list[BaseCreator]
    """
    if uimod.type != ph.UI_TYPE_TRAFFIC_RULES:
        return []

    try:
        (rules_uimod, fusion_uimod, roadway_uimod, transit_uimod, location_uimod, gnss_uimod, cam_uimod,
         term_uimod, leddar_uimod, iit_ca_uimod2, iit_ca_enduser_uimod2) = (
            next(ifilter(lambda x: x.type == ph.UI_TYPE_TRAFFIC_RULES, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_ROADWAY, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_TRANSIT, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_LOCATION, all_used_uimods), None),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_GNSS, all_used_uimods)),
            next(ifilter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods), None),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_TERMINAL, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_LEDDAR, all_used_uimods), None),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_IIT_CA, all_used_uimods), None),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_IIT_CA_END_USER, all_used_uimods), None),
        )

        (wrongway_uimods, wanted_uimods, spotspeed_uimods, sectionspeed_uimods,
         redlight_uimods, rc_uimods, parking_uimods, buslane_uimods, spotspeed_waga_uimods) = (
            filter(lambda x: x.type == ph.UI_TYPE_WRONG_WAY, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_WANTED, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_SPOT_SPEED, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_SECTION_SPEED, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_RED_LIGHT, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_RAILROAD_CROSSING, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_PARKING, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_BUS_LANE, all_used_uimods),
            filter(lambda x: x.type == ph.UI_TYPE_SPOT_SPEED_WAGA, all_used_uimods)
        )

        activate = any([
            uimod_to_newconf_map.get(rules_uimod),
            uimod_to_newconf_map.get(roadway_uimod),
            uimod_to_newconf_map.get(transit_uimod),
            uimod_to_newconf_map.get(location_uimod),
            uimod_to_newconf_map.get(gnss_uimod),
            uimod_to_newconf_map.get(cam_uimod),
            uimod_to_newconf_map.get(term_uimod),
            uimod_to_newconf_map.get(leddar_uimod),
            uimod_to_newconf_map.get(iit_ca_uimod2),
            uimod_to_newconf_map.get(iit_ca_enduser_uimod2),

            any(map(lambda x: uimod_to_newconf_map.get(x), wrongway_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), wanted_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), spotspeed_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), sectionspeed_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), redlight_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), rc_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), parking_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), buslane_uimods)),
            any(map(lambda x: uimod_to_newconf_map.get(x), spotspeed_waga_uimods))
        ])

        if forced or activate:
            creator = ScCreator(
                rules_uimod, roadway_uimod, fusion_uimod, transit_uimod, location_uimod, gnss_uimod, cam_uimod, term_uimod,
                leddar_uimod, wrongway_uimods, wanted_uimods, spotspeed_uimods, sectionspeed_uimods, redlight_uimods,
                rc_uimods,  parking_uimods, buslane_uimods, spotspeed_waga_uimods, iit_ca_uimod2, iit_ca_enduser_uimod2)

            return [creator]

    except StopIteration:
        pass

    return []


class ScCreator(BaseCreator):
    def __init__(self, rules_uimod, roadway_uimod, fusion_uimod, transit_uimod, location_uimod, gnss_uimod, cam_uimod,
                 term_uimod, leddar_uimod, wrongway_uimods, wanted_uimods, spotspeed_uimods, sectionspeed_uimods,
                 redlight_uimods, rc_uimods, parking_uimods, buslane_uimods, spotspeed_waga_uimods,
                 iit_ca_uimod2, iit_ca_enduser_uimod2):
        """
        :type rules_uimod: ph.UiMod
        :type roadway_uimod: ph.UiMod
        :type fusion_uimod: ph.UiMod
        :type transit_uimod: ph.UiMod
        :type location_uimod: ph.UiMod | None
        :type gnss_uimod: ph.UiMod
        :type cam_uimod: ph.UiMod | None
        :type term_uimod: ph.UiMod
        :type leddar_uimod: ph.UiMod | None

        :type wrongway_uimods: list of ph.UiMod
        :type wanted_uimods: list of ph.UiMod
        :type spotspeed_uimods: list of ph.UiMod
        :type sectionspeed_uimods: list of ph.UiMod
        :type redlight_uimods: list of ph.UiMod
        :type rc_uimods: list of ph.UiMod
        :type parking_uimods: list of ph.UiMod
        :type buslane_uimods: list of ph.UiMod
        :type spotspeed_waga_uimods: list of ph.UiMod
        :type iit_ca_uimod2: ph.UiMod2 | None
        :type iit_ca_enduser_uimod2: ph.UiMod2 | None
        """
        BaseCreator.__init__(self)
        self._rules_uimod = rules_uimod
        self._roadway_uimod = roadway_uimod
        self._fusion_uimod = fusion_uimod
        self._transit_uimod = transit_uimod
        self._wrongway_uimods = wrongway_uimods
        self._wanted_uimods = wanted_uimods
        self._spotspeed_uimods = spotspeed_uimods
        self._sectionspeed_uimods = sectionspeed_uimods
        self._redlight_uimods = redlight_uimods
        self._parking_uimods = parking_uimods
        self._buslane_uimods = buslane_uimods
        self._spotspeed_waga_uimods = spotspeed_waga_uimods
        self._location_uimod = location_uimod
        self._gnss_uimod = gnss_uimod
        self._cam_uimod = cam_uimod
        self._leddar_uimod = leddar_uimod
        self._railroad_crossing_uimods = rc_uimods
        self._iit_ca_uimod2 = iit_ca_uimod2
        self._iit_ca_enduser_uimod2 = iit_ca_enduser_uimod2

        self._edge_sc = dg.DgMod.create(
            dg.EDGE_TYPE_SPEEDCONTROL, rules_uimod.pid, rules_uimod.id)
        self._path_to_conf_map = None

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._edge_sc

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        ui_pid = self._rules_uimod.pid

        edge_sc = self._edge_sc  # type: dg.DgMod
        roadway_uiconf = uimod_to_newconf_map.get(self._roadway_uimod) or self._roadway_uimod.load_conf()
        transit_uiconf = uimod_to_newconf_map.get(self._transit_uimod) or self._transit_uimod.load_conf()
        transit_pri = transit_uiconf['ro']
        main_conf = deepcopy(transit_pri.get('main', {}))

        spool_dir = ph.get_spool_dir(ui_pid)

        main_conf['services'] = {
            'camera': self._cam_uimod.conf_path if self._cam_uimod else None,
            'stat': {
                'tout': 60,
                'path': fs.join_path(spool_dir, 'zabbix', 'sc0')
            },
            'gpsd': self._gnss_uimod.conf_path,
            'db.conf': DbUiconf.CONF_PATH,
            'tz.conf': TimeUiconf.CONF_PATH,
            'origin': ph.TermAboutUiconf.get_conf_path(ui_pid),
            'event': {
                'endpoints': [_mk_eventstore_react_path(ui_pid)],
                'args-ua': transit_pri.get('require-args-ua', False),
                'local-arch-base': transit_pri.get('local-arch-base')
            },
            'react': fs.join_path(spool_dir, 'tr{}-sc{}-track'.format(self._fusion_uimod.id, self._rules_uimod.id)),
            'systest': SystestUimod.CONF_PATH,
            'plate-info': '/opt/edge/share/lpr/plate-info.json',
            'vs': transit_pri.get('use-vs', True)
        }

        if len(self._redlight_uimods) == 1:
            rl_uimod = self._redlight_uimods[0]
            red_light_jnode = uimod_to_newconf_map.get(rl_uimod) or rl_uimod.load_conf()
            params_jnode = red_light_jnode['ro'].get('params', {})
            tl_id = params_jnode.get('controller', [{}])[0].get('id', 1)
            red_light_only = params_jnode.get('red-light-only', False)
            main_conf['services']['tl'] = [
                red_light_only,
                fs.join_path(spool_dir, 'tl{}-sc{}-track'.format(0, self._rules_uimod.id)),
                tl_id
            ]
        elif len(self._railroad_crossing_uimods) == 1:
            rc_uimod = self._railroad_crossing_uimods[0]
            railroad_crossing_jnode = uimod_to_newconf_map.get(rc_uimod) or rc_uimod.load_conf()
            params_jnode = railroad_crossing_jnode['ro'].get('params', {})
            tl_id = params_jnode.get('controller', [{}])[0].get('id', 1)
            red_light_only = params_jnode.get('red-light-only', True)
            main_conf['services']['tl'] = [
                red_light_only,
                fs.join_path(spool_dir, 'tl{}-sc{}-track'.format(0, self._rules_uimod.id)),
                tl_id
            ]

        trainer_jnode = transit_pri.get('trainer')
        if trainer_jnode:
            main_conf['services'].update({'trainer': trainer_jnode})

        transit_conf_path = edge_sc.get_conf_path('transit.conf')
        transit_conf = {
            'systest': CtrlUiconf.get_systest(),
            'plate-detected-mode': transit_pri.get("plate-detected-mode", False),
            'name': '',
            'enabled': True,
            'enable-overview': transit_pri.get('enable-overview', False),
            'images': {
                'best': transit_uiconf['rw']['args']['images']
            },
            'rules': [{
                'name': '',
                'enabled': True,
                'limits': {
                    'require-plate': transit_pri.get('require-plate', True)
                }
            }],
        }

        if self._location_uimod:
            main_conf['services']['location'] = self._location_uimod.conf_path

        path_to_conf_map = [
            (edge_sc.get_conf_path('main.conf'), main_conf),
            (edge_sc.get_conf_path('transit.conf'), transit_conf)
        ]

        if self._leddar_uimod:
            transit_conf['le-loop-ref'] = 'loop0'
            transit_conf['rules'][0]['limits']['require-plate'] = False
            main_conf['services']['leddar'] = fs.join_path(spool_dir, 'lg{}-sc{}-track'.format(
                self._leddar_uimod.id, self._rules_uimod.id))

        # add parking rules
        parking_path_to_conf_map = []
        for parking_uimod in self._parking_uimods:
            parking_rule_jnode = uimod_to_newconf_map.get(parking_uimod) or parking_uimod.load_conf()
            k = edge_sc.get_conf_path('parking_{}.conf'.format(parking_uimod.id))
            v = parking_rule_jnode['rw']['args']
            parking_path_to_conf_map.append((k, v))

        for file_path, content in parking_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # # add spot-speed rules
        spotspeed_path_to_conf_map = []
        for spotspeed_uimod in self._spotspeed_uimods:
            spotspeed_rule_jnode = uimod_to_newconf_map.get(spotspeed_uimod) or spotspeed_uimod.load_conf()
            h = spotspeed_rule_jnode['ro'].get('plate-over-ground')
            log = spotspeed_rule_jnode['ro'].get('log-cam-speed', False)

            jnode = deepcopy(spotspeed_rule_jnode['rw']['args'])
            if h is not None:
                jnode['plate-over-ground'] = h
                jnode['log-cam-speed'] = log

            k = edge_sc.get_conf_path('spotspeed_{}.conf'.format(spotspeed_uimod.id))
            spotspeed_path_to_conf_map.append((k, jnode))

        for file_path, content in spotspeed_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # # add spot-speed rules
        spotspeed_waga_path_to_conf_map = []
        for spotspeed_waga_uimod in self._spotspeed_waga_uimods:
            spotspeed_waga_rule_jnode = uimod_to_newconf_map.get(spotspeed_waga_uimod) or spotspeed_waga_uimod.load_conf()
            h = spotspeed_waga_rule_jnode['ro'].get('plate-over-ground')
            log = spotspeed_waga_rule_jnode['ro'].get('log-cam-speed', False)

            jnode = deepcopy(spotspeed_waga_rule_jnode['rw']['args'])
            if h is not None:
                jnode['plate-over-ground'] = h
                jnode['log-cam-speed'] = log

            k = edge_sc.get_conf_path('spotspeed_waga_{}.conf'.format(spotspeed_waga_uimod.id))
            spotspeed_waga_path_to_conf_map.append((k, jnode))

        for file_path, content in spotspeed_waga_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # # add section-speed rules
        sectionspeed_path_to_conf_map = []
        for sectionspeed_uimod in self._sectionspeed_uimods:
            sectionspeed_rule_jnode = uimod_to_newconf_map.get(sectionspeed_uimod) or sectionspeed_uimod.load_conf()
            k = edge_sc.get_conf_path('sectionspeed_{}.conf'.format(sectionspeed_uimod.id))
            jnode = sectionspeed_rule_jnode['rw']['args']
            jnode['ignore-ra'] = sectionspeed_rule_jnode['ro'].get('ignore-ra', False)
            jnode['section']['assert-dist'] = sectionspeed_rule_jnode['ro'].get('assert-dist', 1000)
            sectionspeed_path_to_conf_map.append((k, jnode))

        for file_path, content in sectionspeed_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # add wrong-way rules
        wrongway_path_to_conf_map = []
        for wrongway_uimod in self._wrongway_uimods:
            wrongway_rule_jnode = uimod_to_newconf_map.get(wrongway_uimod) or wrongway_uimod.load_conf()
            k = edge_sc.get_conf_path('wrongway_{}.conf'.format(wrongway_uimod.id))
            v = wrongway_rule_jnode['rw']['args']
            wrongway_path_to_conf_map.append((k, v))

        for file_path, content in wrongway_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # add red-light rules
        redlight_path_to_conf_map = []
        for rl_uimod in self._redlight_uimods:
            red_light_rule_jnode = uimod_to_newconf_map.get(rl_uimod) or rl_uimod.load_conf()
            rl_args_jnode = deepcopy(red_light_rule_jnode['rw']['args'])
            rl_args_jnode['direction'] = red_light_rule_jnode['ro'].get('direction', 'receding')

            k = edge_sc.get_conf_path('redlight_{}.conf'.format(rl_uimod.id))
            redlight_path_to_conf_map.append((k, rl_args_jnode))

        for file_path, content in redlight_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # add railroadcrossing rules
        railroad_crossing_path_to_conf_map = []
        for rc_uimod in self._railroad_crossing_uimods:
            railroad_crossing_rule_jnode = uimod_to_newconf_map.get(rc_uimod) or rc_uimod.load_conf()
            railroad_crossing_args_jnode = deepcopy(railroad_crossing_rule_jnode['rw']['args'])
            railroad_crossing_args_jnode['direction'] = railroad_crossing_rule_jnode['ro'].get('direction', 'receding')

            k = edge_sc.get_conf_path('railroad-crossing_{}.conf'.format(rc_uimod.id))
            railroad_crossing_path_to_conf_map.append((k, railroad_crossing_args_jnode))

        for file_path, content in railroad_crossing_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # add wanted rules
        wanted_path_to_conf_map = []
        for wanted_uimod in self._wanted_uimods:
            wanted_rule_jnode = uimod_to_newconf_map.get(wanted_uimod) or wanted_uimod.load_conf()
            k = edge_sc.get_conf_path('wanted_{}.conf'.format(wanted_uimod.id))
            v = wanted_rule_jnode['rw']['args']
            wanted_path_to_conf_map.append((k, v))

        for file_path, content in wanted_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        # add bus-lane rules
        buslane_path_to_conf_map = []
        for buslane_uimod in self._buslane_uimods:
            buslane_rule_jnode = uimod_to_newconf_map.get(buslane_uimod) or buslane_uimod.load_conf()

            buslane_args_jnode = deepcopy(buslane_rule_jnode['rw']['args'])
            buslane_args_jnode['min-image-dist'] = buslane_rule_jnode['ro'].get('min-image-dist', 1e3)
            k = edge_sc.get_conf_path('buslane_{}.conf'.format(buslane_uimod.id))
            buslane_path_to_conf_map.append((k, buslane_args_jnode))

        for file_path, content in buslane_path_to_conf_map:
            path_to_conf_map.append((file_path, content))

        def get_path_fn(name_conf):
            """
            :type name_conf: (str, str)
            :rtype: str
            """
            return name_conf[0]

        # TODO: separate red-light and railroad-crossing unit
        main_conf['detectors'] = {
            'transit.conf': [transit_conf_path],
            'parking.conf': map(get_path_fn, parking_path_to_conf_map),
            'spot-speed.conf': map(get_path_fn, spotspeed_path_to_conf_map),
            'spot-speed-waga.conf': map(get_path_fn, spotspeed_waga_path_to_conf_map),
            'wrong-way.conf': map(get_path_fn, wrongway_path_to_conf_map),
            'bus-lane.conf': map(get_path_fn, buslane_path_to_conf_map),
            'red-light.conf': map(get_path_fn, redlight_path_to_conf_map + railroad_crossing_path_to_conf_map),
            'railroad-crossing.conf': [],
            # 'red-light.conf': map(get_path_fn, redlight_path_to_conf_map),
            # 'railroad-crossing.conf': map(get_path_fn, railroad_crossing_path_to_conf_map),
            'wanted.conf': map(get_path_fn, wanted_path_to_conf_map),
            'section-speed.conf': map(get_path_fn, sectionspeed_path_to_conf_map)
        }

        ui_road_jnode = roadway_uiconf['rw']['args']
        ui_dir_jnode = ui_road_jnode.get('dir') or dict()
        road_conf_jnode = {
            'road': {
                'dir': {
                    'approaching': ui_dir_jnode.get('approaching'),
                    'receding': ui_dir_jnode.get('receding'),
                    'unknown': ui_dir_jnode.get('unknown')
                }
            },
            'scene-wh': ui_road_jnode['scene-wh']}

        if len(ui_road_jnode['lanes']) > 0:
            road_conf_jnode['road']['lanes'] = ui_road_jnode['lanes']

        main_conf['services'].update({'road.conf': edge_sc.get_conf_path('road.conf')})

        k = edge_sc.get_conf_path('road.conf')
        path_to_conf_map.append((k, road_conf_jnode))

        # add sign
        if self._iit_ca_enduser_uimod2:
            # iit_ca_jnode = uimod_to_newconf_map.get(self._iit_ca_uimod2)
            # iit_ca_uiconf = ph.UiConf2(iit_ca_jnode) if iit_ca_jnode else self._iit_ca_uimod2.uiconf

            iit_ca_enduser_jnode = uimod_to_newconf_map.get(self._iit_ca_enduser_uimod2)
            iit_ca_enduser_uiconf = ph.UiConf2(iit_ca_enduser_jnode) \
                if iit_ca_enduser_jnode else self._iit_ca_enduser_uimod2.uiconf

            iit_ca_enduser_uiconf_conf = iit_ca_enduser_uiconf.pub_conf
            active = iit_ca_enduser_uiconf_conf.get('enabled', False)
            if active:
                works = False

                key_serial = iit_ca_enduser_uiconf.pri['serial']
                key_model = iit_ca_enduser_uiconf.pri['model']
                key_pass = iit_ca_enduser_uiconf.pri['password']

                if key_serial and key_pass and fs.path_exists('/opt/edge/bin/iit-cli'):
                    try:
                        reply = sh.exec_cli('/opt/edge/bin/iit-cli', ['key', 'test-access', key_serial, key_pass],
                                            tout_sec=10, encoding='utf-8')

                        works = 'PASSED' in reply
                    except TimeoutError:
                        pass
                    except InternalError:
                        pass

                if not works:
                    # disable for next time
                    iit_ca_enduser_uiconf_conf['enabled'] = False
                    iit_ca_enduser_uiconf.set_pub_conf(iit_ca_enduser_uiconf_conf)
                    self._iit_ca_enduser_uimod2.save_uiconf(iit_ca_enduser_uiconf)
                else:
                    iit_conf_jnode = {
                        'key-sign-pass': key_pass,
                        'key-model': key_model,
                        'key-serial': key_serial,
                        'hash-algo': 'gost34311',
                        'sign-algo': 'dstu4145',
                        'append-cert': iit_ca_enduser_uiconf.pri.get('append-cert', False)
                    }

                    iit_conf_filepath = edge_sc.get_conf_path('sign.iit.conf')

                    path_to_conf_map.append((iit_conf_filepath, iit_conf_jnode))

                    main_conf['services']['sign'] = {
                        'provider': {
                            'iit': iit_conf_filepath
                        }
                    }

        self._path_to_conf_map = path_to_conf_map

    def complete(self):
        path_to_conf_map = self._path_to_conf_map  # type: dict

        edge_sc = self._edge_sc  # type: dg.DgMod
        edge_sc.install()
        edge_sc.save_setupstore_conf(active=True)

        for conf_path, content in path_to_conf_map:
            fs.write_json_file(content, conf_path, pretty=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class ScDeleter:
    def __init__(self, dgmod):
        """
        :type dgmod: dg.DgMod
        """
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: ScDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_TRAFFIC_RULES:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_SPEEDCONTROL, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self):
        self._dgmod.uninstall()
