from itertools import ifilter

from edge.com import fs
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator
from edge.webserver.sysconf.uiconf.constroller import CtrlUiconf


# noinspection PyUnusedLocal
def tr_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if entry_uimod.type != ph.UI_TYPE_FUSION:
        return []

    try:
        ra_uimod, cam_uimod, rstreaming_uimod, hstreaming_uimod, transit_uimod, fusion_uimod, rules_uimod, lpr_uimod, location_uimod = (
            next(ifilter(lambda x: x.type == ph.UI_TYPE_RADAR, all_used_uimods), None),
            next(ifilter(lambda x: x.type in [ph.UI_TYPE_MV_CAMERA, ph.UI_TYPE_RTSP_CAMERA], all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_RTSP_STREAMING, all_used_uimods), None),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_HTTP_STREAMING, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_TRANSIT, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_FUSION, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_TRAFFIC_RULES, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_LPR, all_used_uimods)),
            next(ifilter(lambda x: x.type == ph.UI_TYPE_LOCATION, all_used_uimods), None))

        activate = any([
            uimod_to_newconf_map.get(cam_uimod),
            uimod_to_newconf_map.get(hstreaming_uimod),
            uimod_to_newconf_map.get(transit_uimod),
            uimod_to_newconf_map.get(rstreaming_uimod),
            uimod_to_newconf_map.get(location_uimod)
        ])

        if forced or activate:
            creator = TrCreator(
                ra_uimod, cam_uimod, lpr_uimod, rstreaming_uimod, hstreaming_uimod,
                rules_uimod, transit_uimod, fusion_uimod, location_uimod)

            return [creator]

    except StopIteration:
        pass

    return []


class TrCreator(BaseCreator):
    def __init__(self, ra_uimod, cam_uimod, lpr_uimod, rstreaming_uimod, hstreaming_uimod,
                 rules_uimod, transit_uimod, fusion_uimod, location_uimod):
        """
        :type ra_uimod: ph.UiMod
        :type cam_uimod: ph.UiMod
        :type lpr_uimod: ph.UiMod
        :type rstreaming_uimod: ph.UiMod | None
        :type hstreaming_uimod: ph.UiMod
        :type rules_uimod: ph.UiMod
        :type transit_uimod: ph.UiMod
        :type fusion_uimod: ph.UiMod
        :type location_uimod: ph.Uimod | None
        """
        BaseCreator.__init__(self)
        self._ra_uimod = ra_uimod
        self._cam_uimod = cam_uimod
        self._lpr_uimod = lpr_uimod
        self._hstreaming_uimod = hstreaming_uimod
        self._rstreaming_uimod = rstreaming_uimod
        self._rules_uimod = rules_uimod
        self._transit_uimod = transit_uimod
        self._fusion_uimod = fusion_uimod
        self._location_uimod = location_uimod

        self._tr_dgmod = dg.DgMod.create(dg.EDGE_TYPE_TRACKER, self._fusion_uimod.pid, self._fusion_uimod.id)
        self._main_misc = None

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._tr_dgmod

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """

        cam_uiconf = uimod_to_newconf_map.get(self._cam_uimod) or self._cam_uimod.load_conf()
        hstreaming_uiconf = uimod_to_newconf_map.get(self._hstreaming_uimod) or self._hstreaming_uimod.load_conf()
        hstreaming_pub = hstreaming_uiconf['rw']
        transit_uiconf = uimod_to_newconf_map.get(self._transit_uimod) or self._transit_uimod.load_conf()

        tr_dgmod = self._tr_dgmod
        spool_dir = ph.get_spool_dir(self._fusion_uimod.pid)
        if self._cam_uimod.type == ph.UI_TYPE_MV_CAMERA:
            cam_pri = cam_uiconf['ro']
            if cam_pri.get('raw-format', 'MONO8T') == 'MONO8T':
                sourse_dir = fs.join_path(spool_dir, 'mg{}-tr{}-scene'.format(
                    self._cam_uimod.id, self._fusion_uimod.id))
            else:
                sourse_dir = fs.join_path(spool_dir, 'rc{}-tr{}-scene'.format(
                    self._cam_uimod.id, self._fusion_uimod.id))

        elif self._cam_uimod.type == ph.UI_TYPE_RTSP_CAMERA:
            sourse_dir = fs.join_path(spool_dir, 'rg{}-tr{}-scene'.format(
                self._cam_uimod.id, self._fusion_uimod.id))

        else:
            raise NotImplementedError

        plate_detected_mode = transit_uiconf['ro'].get("plate-detected-mode", False)

        # create main.conf
        main_conf = {
            "systest": CtrlUiconf.get_systest(),
            "plate-detected-mode": plate_detected_mode,
            "Monitoring": {
                "StatFile": fs.join_path(spool_dir, 'zabbix', 'tr0'),
                "StatTout": 5
            },
            "FileApi": {
                "AddTrackReact": {
                    "Dir": fs.join_path(spool_dir, 'lr{}-tr{}-track'.format(
                        self._lpr_uimod.id, self._fusion_uimod.id))
                },
                "AddSceneReact": {
                    "MgDir": sourse_dir,
                    "LrDir": fs.join_path(spool_dir, 'lr{}-tr{}-scene'.format(
                        self._lpr_uimod.id, self._fusion_uimod.id)),
                    "Tout": 750
                },
                "TrackEvent": {
                    "endpoints": [
                        fs.join_path(spool_dir, 'tr{}-sc{}-track'.format(
                            self._fusion_uimod.id, self._rules_uimod.id))
                    ],
                    "debounce-tout": 0 if plate_detected_mode else 3  # TODO: add tout when refactor debouncer
                },
                "SceneEvent": []
            },
            'store': {
                "max-file-count": 20,  # max-file-count >> debounce-tout * lr_track_fps (or we'll lose some events)
                "endpoint": fs.join_path(spool_dir, 'tr{}-store'.format(tr_dgmod.id))
            },
            'services': {
                'camera': self._cam_uimod.conf_path
            }
        }

        if self._location_uimod:
            main_conf['services']['location'] = self._location_uimod.conf_path
            location_uiconf = uimod_to_newconf_map.get(self._location_uimod) or self._location_uimod.load_conf()
            main_conf['ra-vehicle-ltrb'] = location_uiconf['ro'].get('ra-vehicle-ltrb', [1, 2, 1, 0])

        # enable http streaming
        if hstreaming_pub['enabled']:
            # noinspection PyUnresolvedReferences
            main_conf['FileApi']['SceneEvent'].append({
                'SkipRate': hstreaming_pub['live']['videostream']['skip-rate'],
                'Dir': fs.join_path(spool_dir, 'tr{}-or{}-scene'.format(
                    self._fusion_uimod.id, self._hstreaming_uimod.id))
            })

        # enable rtsp-streaming
        if self._rstreaming_uimod:
            rstreaming_uiconf = uimod_to_newconf_map.get(self._rstreaming_uimod) or self._rstreaming_uimod.load_conf()
            rstreaming_pub = rstreaming_uiconf.get('rw')
            if rstreaming_pub['enabled']:
                # noinspection PyUnresolvedReferences
                main_conf['FileApi']['SceneEvent'].append({
                    'SkipRate': rstreaming_uiconf['rw']['live']['videostream']['skip-rate'],
                    'Dir': fs.join_path(spool_dir, 'tr{}-or{}-scene'.format(
                        self._fusion_uimod.id, self._rstreaming_uimod.id))
                })

        # add ra-dir for speed pipelines
        if self._ra_uimod:
            main_conf['FileApi']['AddSceneReact']['RaDir'] = fs.join_path(
                spool_dir, 'ra{}-tr{}-scene'.format(
                    self._ra_uimod.id, self._fusion_uimod.id))

        # create misc.conf
        misc_conf = {
            "min_valid_radar_track_length": 5,
            "lpr_track_caching_time": 4,
            "radar_track_caching_time": 3
        }

        self._main_misc = main_conf, misc_conf

    def complete(self):
        main_conf, misc_conf = self._main_misc  # type: dict, dict

        edge_tr = self._tr_dgmod
        edge_tr.install()
        edge_tr.save_main_conf(main_conf)
        edge_tr.save_misc_conf(misc_conf)
        edge_tr.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_dgmod(self.dgmod.type, self.dgmod.id)


class TrDeleter:
    def __init__(self, dgmod):
        """
        :type dgmod: dg.DgMod
        """
        self._dgmod = dgmod

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: TrDeleter | None
        """
        if entry_uimod.type == ph.UI_TYPE_FUSION:
            dgmod = dg.DgMod.create(dg.EDGE_TYPE_TRACKER, entry_uimod.pid, entry_uimod.id)
            return cls(dgmod)

    def run(self):
        self._dgmod.uninstall()
