# coding=utf-8

from edge.com import fs
from edge.webserver.sysconf import ph
from edge.webserver.sysconf.dgmod import dg
from edge.webserver.sysconf.dgmod.creator import BaseCreator
from edge.webserver.sysconf.uiconf.terminal import TerminalUiconf


# noinspection PyUnusedLocal
def ws_get_creators(entry_uimod, all_used_uimods, uimod_to_newconf_map, forced=False):
    """
    :type entry_uimod: ph.UiMod | ph.UiMod2
    :type all_used_uimods: set[ph.UiMod, ph.UiMod2]
    :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
    :type uimod_to_newconf_map: dict
    :type forced: bool
    :rtype: list of BaseCreator
    """
    if entry_uimod.type != ph.UI_TYPE_WEBSERVER:
        return []

    try:
        terminal_uimods = filter(lambda x: x.type == ph.UI_TYPE_TERMINAL, all_used_uimods)
        hstreaming_uimods = filter(lambda x: x.type == ph.UI_TYPE_HTTP_STREAMING, all_used_uimods)
        assert len(hstreaming_uimods) == len(terminal_uimods)

        # TODO проверить что ref terminal коррелирует с ref http-streaming ?
        terminal_uimods.sort(key=lambda item: int(item.id))
        hstreaming_uimods.sort(key=lambda item: int(item.id))

        activate = any([
            any(map(lambda x: uimod_to_newconf_map.get(x), hstreaming_uimods))
        ])

        if forced or activate:
            return [WsCreator(entry_uimod, terminal_uimods, hstreaming_uimods)]

    except StopIteration:
        pass

    return []


class WsCreator(BaseCreator):
    def __init__(self, ui_entry, terminal_uimods, hstreaming_uimods):
        """
        :type ui_entry: ph.UiMod
        :type terminal_uimods: list of ph.UiMod
        :type hstreaming_uimods: list of ph.UiMod
        """
        BaseCreator.__init__(self)
        self._ui_entry_point = ui_entry
        self._main_conf = None
        self._hstreaming_uimods = hstreaming_uimods
        self._terminal_uimods = terminal_uimods
        self._edge_ws = dg.CTRL_WS_DGMOD

    @property
    def dgmod(self):
        """
        :rtype: dg.DgMod
        """
        return self._edge_ws

    def prepare(self, uimod_to_newconf_map):
        """
        :param uimod_to_newconf_map: Dict[UiMod | UiMod2, Jnode]
        :type uimod_to_newconf_map: dict
        """
        edge_ws = self._edge_ws
        main_conf = edge_ws.load_main_conf()

        media_jnode_list = []
        for index in range(0, len(self._terminal_uimods)):
            hstreaming_uimod = self._hstreaming_uimods[index]
            term_uimod = self._terminal_uimods[index]

            ui_pid = term_uimod.pid
            hstreaming_uiconf = uimod_to_newconf_map.get(hstreaming_uimod) or hstreaming_uimod.load_conf()
            spool_dir = ph.get_spool_dir(ui_pid)

            origin_uuid = TerminalUiconf(term_uimod.conf_path).get_uuid()
            origin_jnode = {
                'origin': origin_uuid
            }

            if hstreaming_uiconf['rw']['enabled']:
                origin_jnode['LiveEndpoint'] = fs.join_path(spool_dir, 'or{}-ws0-scene'.format(hstreaming_uimod.id))

            media_jnode_list.append(origin_jnode)

        main_conf['Media'] = media_jnode_list
        self._main_conf = main_conf

    def complete(self):
        main_conf = self._main_conf  # type: dict

        edge_ws = self._edge_ws
        edge_ws.install()
        edge_ws.save_main_conf(main_conf)
        edge_ws.save_setupstore_conf(active=True)

    def rollback(self):
        pass

    def notify(self):
        """
        :rtype: None
        """
        dg.LifeCycle.restart_ws_dgmod()


class WsDeleter:
    def __init__(self, origin_uuid):
        """
        :type origin_uuid: str
        """
        self._term_uuid = origin_uuid

    @classmethod
    def create(cls, entry_uimod):
        """
        :type entry_uimod: ph.UiMod
        :rtype: WsDeleter
        """
        if entry_uimod.type == ph.UI_TYPE_FACTORY:
            term_uimod = ph.UiMod.create_term(entry_uimod.pid)
            term_uuid = TerminalUiconf(term_uimod.conf_path).get_uuid()
            return cls(term_uuid)

    def run(self):
        dgmod = dg.CTRL_WS_DGMOD
        dgconf = dgmod.load_main_conf()
        dgconf['Media'] = filter(lambda x: x['origin'] != self._term_uuid, dgconf['Media'])
        dgmod.save_main_conf(dgconf)
