from datetime import datetime, tzinfo
from edge import eventlog
from edge.com import fs


# noinspection PyClassHasNoInit
class BsDgstats:
    _64K_PATH = '/var/spool/edge/system/zabbix/bs65535'

    @staticmethod
    def get_chan_stats(stats_path, chanref, tz=None):
        """
        :type stats_path: str
        :type chanref: str
        :type tz: tzinfo | None
        :rtype: dict | None
        """
        dgstats = fs.load_json3(stats_path, default=None)
        if dgstats:
            dgstats_stat = dgstats['stat']  # type: dict
            dgstats_mtime = dgstats_stat['mtime']  # type: tuple
            chans = filter(lambda x: x['channel'] == chanref, dgstats['storage'])
            for chan in chans:
                min_time = chan['min-time']  # type: tuple
                max_time = chan['max-time']  # type: tuple
                return {
                    'time': datetime.fromtimestamp(dgstats_mtime[0], tz).isoformat(),
                    'min-time': datetime.fromtimestamp(min_time[0], tz).isoformat(),
                    'max-time': datetime.fromtimestamp(max_time[0], tz).isoformat(),
                    'df': chan['df'],
                    'du': chan['du'],
                    'diru': chan['diru']
                }

        eventlog.log_w('fail: NotFoundError')

    @classmethod
    def get_chan_stats_64k(cls, chanref, tz=None):
        """
        :type chanref: str
        :type tz: tzinfo | None
        :rtype: dict | None
        """
        return cls.get_chan_stats(cls._64K_PATH, chanref, tz)
