# coding=utf-8
__all__ = [
    'IitCaUiconf',
    'IitCaEndUserUiconf'
]

from edge.webserver.sysconf import ph
from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2
from edge import eventlog


class IitCaUiconf(UiConf2):
    def __init__(self, conf):
        """
        :type conf: UiConf2
        """
        UiConf2.__init__(self, conf.raw)

    @classmethod
    def create(cls, factory_conf=None):
        """
        :type factory_conf: dict | None
        :rtype: IitCaUiconf
        """
        service_uiref = factory_conf['ref']
        iit_ca_unit_name = u'ІІТ ЦСК'

        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/iit-ca-schema.json',
                'lock-path': '/var/run/edge/{}'.format(service_uiref)
            },
            'pub': {
                'type': ph.UI_TYPE_IIT_CA,
                'conf': {
                    'name': iit_ca_unit_name,
                    'proxy': {
                        'enable': True,
                        'host': '10.100.27.101',
                        'port': 3128,
                        'auth-type': 'anonymous',
                        'auth-user': '',
                        'auth-pass': ''
                    },
                    'cmp': {
                        'host': 'ca.mvs.gov.ua',
                        'port': 80
                    },
                    'tsp': {
                        'host': 'ca.mvs.gov.ua',
                        'port': 80
                    },
                    'crl': {
                        'enable': False,
                        'full-and-delta-crls': False,
                        'only-own-crls': False
                    },
                    'ocsp': {
                        'enable': False,
                        'host': '',
                        'port': 80
                    }
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def proxy_pass(self):
        """
        :rtype: str | None
        """
        return self.pub_conf['proxy'].get('auth-pass')


class IitCaEndUserUiconf(UiConf2):
    def __init__(self, conf):
        """
        :type conf: UiConf2
        """
        UiConf2.__init__(self, conf.raw)

    @classmethod
    def create(cls, factory_jnode=None):
        """
        :type factory_jnode: dict | None
        :rtype: IitCaEndUserUiconf
        """
        name = factory_jnode.get('name', u'ІІТ Користувач ЦСК')
        factory_conf = factory_jnode['conf']
        media_conf = factory_conf['media']
        model = media_conf['model']
        media_serial = media_conf['serial']
        media_pass = media_conf['password']
        iit_ca_uimod_ref = factory_conf['parent-ref']
        zip_file_name = factory_conf.get('zip-name', '')
        sign_active = factory_conf.get('active', False)

        conf_jnode = {
            'pri': {
                'model': model,
                'serial': media_serial,
                'password': media_pass,
                'lock-path': '/var/run/edge/{}'.format(iit_ca_uimod_ref),
                'csr-name': zip_file_name
            },
            'pub': {
                'type': ph.UI_TYPE_IIT_CA_END_USER,
                'conf': {
                    'name': name,
                    'enabled': sign_active
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def model(self):
        """
        :rtype: str
        """
        return self.pri['model']

    @property
    def serial(self):
        """
        :rtype: str
        """
        return self.pri['serial']

    @property
    def password(self):
        """
        :rtype: str
        """
        return self.pri['password']

    @property
    def csr_name(self):
        """
        :rtype: str
        """
        return self.pri.get('csr-name', '')

    @property
    def crt_stats(self):
        """
        :rtype: dict | None
        """
        return self.pri.get('crt')

    def set_crt_stats(self, stats):
        """
        :type stats: dict
        :rtype: None
        """
        self.pri['crt'] = stats

