import os
import commands
import ujson

from edge import eventlog
from edge.com import net

UI_NETWORK_JNODE_FILEPATH = '/etc/edge/setupstore/admin/actions/network.json'
MANAGEMENT_MAC_FILEPATH = '/etc/network/management-mac'


# noinspection PyClassHasNoInit
class RtspStreamingUiconf:
    def __init__(self, ui_rw_jnode):
        """
        :type ui_rw_jnode: dict
        """
        self._ui_rw_jnode = ui_rw_jnode

    @property
    def rtsp_url(self):
        """
        :rtype: str
        """

        try:
            with open(UI_NETWORK_JNODE_FILEPATH) as json_file:
                ui_network_jnode = ujson.load(json_file)
            mac_address = ui_network_jnode['ro']['mac']

        except IOError as x:
            if x.errno == 2:
                if os.path.exists(MANAGEMENT_MAC_FILEPATH):
                    with open(MANAGEMENT_MAC_FILEPATH, 'r') as mgmt_mac_file:
                        mac_address = mgmt_mac_file.readlines()[0].strip()
                else:
                    raise

            else:
                eventlog.log_d('fail: read network settings')
                raise IOError

        ifname = net.get_ifname(mac_address)  # TODO: check if ifname is not None
        try:
            ip_addr = commands.getoutput("ip addr show %s | grep -v '127.0.0.1' | grep -w inet | awk '{print $2}'"
                                         % ifname).split('/')[0]
        except Exception:
            eventlog.log_d('fail: discover ip-address')
            raise RuntimeError

        tcp_port = str(self._ui_rw_jnode['streaming']['tcp-port'])
        ui_rtsp_codec = self._ui_rw_jnode['live']['videostream']['format']
        if ui_rtsp_codec == 'mjpeg':
            rtsp_codec = 'mjpeg'
        elif ui_rtsp_codec == 'h264-cqp' or 'h264-cbr':
            rtsp_codec = 'h264'
        else:
            eventlog.log_d('unknown rtsp-codec %s' % ui_rtsp_codec)
            raise ValueError

        rtsp_url = 'rtsp://' + ip_addr + ':' + tcp_port + '/live-' + rtsp_codec

        return rtsp_url
