import os
from edge.com import fs


# noinspection PyClassHasNoInit
class TimeUiconf:
    CONF_PATH = '/etc/edge/setupstore/admin/actions/time.json'
    SCHEMA_PATH = '/etc/edge/setupstore/schemas/time.json'
    SCHEMA_MANUAL_PATH = '/opt/edge/share/setupstore/ui/time-schema-manual.json'
    DEFCONF_PATH = '/opt/edge/share/setupstore/ui/time-jnode.json'

    @classmethod
    def get_tz(cls):
        """
        :rtype: pytz.datetime.tzinfo | None
        """
        uiconf = fs.load_json3(cls.CONF_PATH, default=None)
        if uiconf:
            uiconf_pub = uiconf['rw']  # type: dict
            uiconf_args = uiconf_pub['args']  # type: dict

            import pytz
            tz_name = uiconf_args.get('tz')
            return pytz.timezone(tz_name)

    @classmethod
    def install(cls):
        fs.create_jnode_from_default(cls.DEFCONF_PATH, cls.CONF_PATH)

        if not os.path.islink(TimeUiconf.SCHEMA_PATH):
            fs.ln_schema(TimeUiconf.SCHEMA_PATH, TimeUiconf.CONF_PATH)

        elif not os.path.exists(os.readlink(TimeUiconf.SCHEMA_PATH)):
            fs.ln_schema(TimeUiconf.SCHEMA_PATH, TimeUiconf.CONF_PATH)

    @classmethod
    def load_conf(cls, **kwargs):
        """
        :rtype: dict
        :raise FsError:
        """
        return fs.load_json3(cls.CONF_PATH, **kwargs)

    @classmethod
    def save_conf(cls, conf_jnode):
        """
        :type conf_jnode: dict
        :rtype: None
        """
        fs.write_json_file(conf_jnode, cls.CONF_PATH, pretty=True)
