from edge.com import fs
from edge.exceptions import InternalError


class UiConf2:
    def __init__(self, jnode):
        """
        :type jnode: Dict
        """
        self._raw = jnode

    @property
    def raw(self):
        """
        :rtype: dict
        """
        return self._raw

    @property
    def pri(self):
        """
        :rtype: dict
        """
        return self.raw['pri']

    @property
    def pub(self):
        """
        :rtype: dict
        """
        return self.raw['pub']

    @property
    def pub_conf(self):
        """
        :rtype: dict
        """
        return self.pub['conf']

    @property
    def pub_name(self):
        """
        :rtype: str
        """
        return self.pub_conf['name']

    def set_pub(self, jnode):
        """
        :type jnode: dict
        :rtype: None
        """
        self.raw['pub'] = jnode

    def set_pri(self, jnode):
        """
        :type jnode: dict
        :rtype: None
        """
        self.raw['pri'] = jnode

    def set_ver(self, version):
        """
        :type version: str
        :rtype: None
        """
        self.pri['ver'] = version

    @property
    def ver(self):
        """
        :rtype: str
        """
        ver = self.pri.get('ver')
        if ver is None:
            raise InternalError
        return ver

    def set_pub_conf(self, jnode):
        """
        :type jnode: dict
        :rtype: bool
        """
        changed = self.pub['conf'] != jnode
        self.pub['conf'] = jnode
        return changed

    def set_pub_name(self, name):
        """
        :type name: str
        :rtype: bool
        """
        changed = self.pub_conf['name'] != name
        self.pub_conf['name'] = name
        return changed

    @property
    def lock_path(self):
        """
        :rtype: str
        """
        return self.pri['lock-path']

    @property
    def _schema_path(self):
        """
        :rtype: str | None
        """
        return self.pri.get('schema-path')

    @property
    def terminal_ref(self):
        """
        :rtype: str | None
        """
        return self.pri.get('terminal-ref')

    def validate(self):
        """
        :rtype: None
        :raises ParamsError
        """
        schema_path = self._schema_path
        if schema_path:
            fs.validate_json(self.pub, schema_path)
