__all__ = [
    'WiegandPcbUiconf',
    'WiegandPcbInputPortUiconf',
    'WiegandPcbRelayPortUiconf'
]

from edge.webserver.sysconf.uiconf.uiconf2 import UiConf2
from edge import eventlog


class WiegandPcbUiconf(UiConf2):
    def __init__(self, conf):
        """
        :type conf: UiConf2
        """
        UiConf2.__init__(self, conf.raw)

    @classmethod
    def create(cls, factory_conf=None):
        """
        :type factory_conf: dict | None
        :rtype: WiegandPcbUiconf
        """
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb-schema.json',
                'lock-path': factory_conf['lock-path'],
                'endpoint': factory_conf['endpoint']
            },
            'pub': {
                'type': 'wiegand-pcb',
                'conf': {
                    'name': 'Wiegand PCB'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def endpoint(self):
        """
        :rtype: str
        """
        return self.pri['endpoint']


class WiegandPcbInputPortUiconf(UiConf2):
    def __init__(self, conf):
        """
        :type conf: UiConf2
        """
        UiConf2.__init__(self, conf.raw)

    @classmethod
    def create(cls, unit_chan):
        """
        :type unit_chan: int
        :rtype: WiegandPcbUiconf
        """
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb~input-port-schema.json',
                'address': unit_chan + 2
            },
            'pub': {
                'type': 'wiegand-pcb~input-port',
                'conf': {
                    'name': 'IN{}'.format(unit_chan + 1),
                    'active': 'open'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def ro_bit_address(self):
        """
        :rtype: int
        """
        return self.pri['address']

    @property
    def ro_bit_active_state(self):
        """
        :rtype: bool
        """
        ui_active = self.pub['conf']['active']
        if ui_active == 'open':
            active = True
        elif ui_active == 'closed':
            active = False
        else:
            eventlog.log_e('fail: unsupported')
            raise NotImplementedError

        return active


class WiegandPcbRelayPortUiconf(UiConf2):
    def __init__(self, conf):
        """
        :type conf: UiConf2
        """
        UiConf2.__init__(self, conf.raw)

    @classmethod
    def create(cls, unit_chan):
        """
        :type unit_chan: int
        :rtype: WiegandPcbUiconf
        """
        conf_jnode = {
            'pri': {
                'schema-path': '/opt/edge/share/setupstore/ui/wiegand-pcb~relay-port-schema.json',
                'address': unit_chan
            },
            'pub': {
                'type': 'wiegand-pcb~relay-port',
                'conf': {
                    'name': 'OUT{}'.format(unit_chan + 1),
                    'active': 'open'
                }
            }
        }

        return cls(UiConf2(conf_jnode))

    @property
    def rw_bit_address(self):
        """
        :rtype: int
        """
        return self.pri['address']

    @property
    def rw_bit_active_state(self):
        """
        :rtype: bool
        """
        ui_active = self.pub['conf']['active']
        if ui_active == 'open':
            active = True
        elif ui_active == 'closed':
            active = False
        else:
            eventlog.log_e('fail: unsupported')
            raise NotImplementedError

        return active
