from edge.com import fs
from edge.webserver.sysconf.ph import CTRL_UICONF_DIR


# noinspection PyClassHasNoInit
class SignUimod:
    ref = 'sign.0'
    _DEF_UICONF_PATH = '/opt/edge/share/setupstore/ui/sign-jnode.json'
    _DEF_UISCHEMA_PATH = '/opt/edge/share/setupstore/ui/sign-schema.json'

    _UISCHEMA_PATH = '/etc/edge/setupstore/schemas/sign-schema.json'
    _UICONF_PATH = '/etc/edge/setupstore/admin/actions/sign/0.json'

    KEY_PATH = fs.join_path(CTRL_UICONF_DIR, 'sign', 'sign.key')
    CRT_PATH = fs.join_path(CTRL_UICONF_DIR, 'sign', 'sign.crt')

    @classmethod
    def load_conf(cls, **kwargs):
        """
        :rtype: dict
        """
        return fs.load_json3(cls._UICONF_PATH, **kwargs)

    @classmethod
    def save_conf(cls, conf_jnode):
        """
        :type conf_jnode: dict
        :rtype: None
        """
        fs.write_json_file(conf_jnode, cls._UICONF_PATH, pretty=True)

    @classmethod
    def install(cls):
        """
        :rtype: None
        """
        if not fs.path_exists(cls._UICONF_PATH):
            fs.silent_cp(cls._DEF_UICONF_PATH, cls._UICONF_PATH)

        if not fs.path_exists(cls._UISCHEMA_PATH):
            fs.silent_link(cls._DEF_UISCHEMA_PATH, cls._UISCHEMA_PATH)

    @classmethod
    def uninstall(cls):
        """
        :rtype: None
        """
        fs.silent_rm_r(cls._UICONF_PATH)
        fs.silent_rm_r(cls.KEY_PATH)
        fs.silent_rm_r(cls.CRT_PATH)
        fs.silent_rm_r(cls._UISCHEMA_PATH)

    @classmethod
    def install_key_crt(cls, crt_path, key_path, key_pass):
        """
        :type key_path: str
        :type crt_path: str
        :type key_pass: str | None
        :rtype: dict
        """
        uiconf = cls.load_conf()

        fs.silent_cp(key_path, cls.KEY_PATH)
        fs.silent_cp(crt_path, cls.CRT_PATH)

        uiconf_pri = uiconf['ro']
        uiconf_pri['key'] = True
        uiconf_pri['crt'] = True
        if key_pass:
            uiconf_pri['key-sign-pass'] = key_pass
        else:
            uiconf_pri['key-sign-pass'] = ''

        cls.save_conf(uiconf)
        return uiconf

    @classmethod
    def schema_path(cls):
        """
        :rtype: str
        """
        return cls._UISCHEMA_PATH
