import iso8601
from .test import Test, TestStep, Summary, TestArgsError

from edge.webserver.sysconf import ph
from edge.webserver import cgi
from edge import eventlog


class DatetimeTestStep(TestStep):
    def __init__(self, services, session_ref, test_ref, step_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        :type step_ref: str
        """
        TestStep.__init__(self, services, session_ref, test_ref, step_ref)

    def _edit_log_item_jnode(self, log_item_jnode, body_jnode):
        """
        :type log_item_jnode: dict
        :type body_jnode: dict
        :rtype: dict
        """
        # unpack #/data/report
        report = body_jnode.get('report', False)
        if type(report) != bool:
            eventlog.log_i('fail: http400. param #/report has invalid value')
            raise cgi.CgiError400

        try:
            data_jnode = body_jnode['data']
            time_jnode = data_jnode['time']

            # unpack #/data/time/actual
            if time_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/time/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/time/expected
            expected_time = time_jnode.get('expected')
            if expected_time:
                iso8601.parse_date(expected_time)

            # unpack #/summary
            summary_jnode = Summary.repack(body_jnode.get('summary'))

            # apply changes
            log_item_jnode['report'] = report
            log_item_jnode['data']['time']['expected'] = expected_time
            log_item_jnode['summary'] = summary_jnode
            return log_item_jnode

        except (KeyError, ValueError, iso8601.ParseError) as x:
            eventlog.log_i('fail: http400 <{}>'.format(x.message))
            raise cgi.CgiError400


class DatetimeTest(Test):
    def __init__(self, services, session_ref, test_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        """
        Test.__init__(self, services, session_ref, test_ref)

    def _mk_step_jnode(self, test_jnode, step_jnode, args_jnode):
        """
        :type test_jnode: dict
        :type step_jnode: dict
        :type args_jnode: None
        """
        if args_jnode:
            eventlog.log_i('fail: http400')
            raise TestArgsError

        term_uuid = test_jnode['ro']['origin']
        term_pid = ph.RootInfoUiconf.get_term_pid(term_uuid)
        if term_pid is None:
            raise NotImplementedError

        blob_ts, blob_ref = self._get_live_image(term_pid)
        step_jnode['data'] = {
            'time': {
                'actual': self._services.tz.get_datetime_isostr(blob_ts / 1e6)
            },
            'images': {
                'scene': {
                    'ref': blob_ref
                }
            }
        }

        return step_jnode
