import gps

from .test import Test, TestStep, Summary, TestArgsError

from edge import eventlog
from edge.webserver import cgi


def _get_geodata():
    """
    :rtype: (str, str) | None
    """
    session = gps.gps(host='localhost', port='2947', mode=gps.WATCH_ENABLE)

    try_count = 0
    while True:
        last_msg = session.next()
        msg_class = last_msg.get('class')
        if msg_class == "DEVICES":
            if len(last_msg.get('devices', [])) == 0:
                eventlog.log_i("fail: gps absent")
                return None
        elif msg_class == "TPV":
            if session.fix.mode in [gps.MODE_2D, gps.MODE_3D]:
                break

        # TODO: Return something to frontend when no GPS fix
        try_count += 1
        if try_count > 10:
            eventlog.log_i('fail: no fix')
            return None

    latitude = session.fix.latitude
    longitude = session.fix.longitude

    eventlog.log_i("%s %s" % (latitude, longitude))
    return latitude, longitude


class GeolocationTestStep(TestStep):
    def __init__(self, services, session_ref, test_ref, step_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        :type step_ref: str
        """
        TestStep.__init__(self, services, session_ref, test_ref, step_ref)

    def _edit_log_item_jnode(self, log_item_jnode, body_jnode):
        """
        :type log_item_jnode: dict
        :type body_jnode: dict
        :rtype: dict
        """
        # unpack #/data/report
        report = body_jnode.get('report', False)
        if type(report) != bool:
            eventlog.log_i('fail: http400. param #/report has invalid value')
            raise cgi.CgiError400

        try:
            data_jnode = body_jnode['data']
            lat_jnode = data_jnode['latitude']
            long_jnode = data_jnode['longitude']

            # unpack #/data/latitude/actual
            if lat_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/latitude/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/latitude/expected
            expected_lat = lat_jnode.get('expected')
            if expected_lat is not None:
                if type(expected_lat) not in [float, int]:
                    eventlog.log_i('fail: http400. param #/data/latitude/expected has invalid value')
                    raise cgi.CgiError400

            # unpack #/data/longitude/actual
            if long_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/longitude/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/longitude/expected
            expected_long = long_jnode.get('expected')
            if expected_long is not None:
                if type(expected_long) not in [float, int]:
                    eventlog.log_i('fail: http400. param #/data/longitude/expected has invalid value')
                    raise cgi.CgiError400

            # unpack #/summary
            summary_jnode = Summary.repack(body_jnode.get('summary'))

            # apply changes
            log_item_jnode['report'] = report
            log_item_jnode['data']['latitude']['expected'] = expected_lat
            log_item_jnode['data']['longitude']['expected'] = expected_long
            log_item_jnode['summary'] = summary_jnode
            return log_item_jnode

        except (KeyError, ValueError) as x:
            eventlog.log_i('fail: http400 <{}>'.format(x.message))
            raise cgi.CgiError400


class GeolocationTest(Test):
    def __init__(self, services, session_ref, test_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        """
        Test.__init__(self, services, session_ref, test_ref)

    def _mk_step_jnode(self, test_jnode, step_jnode, args_jnode):
        """
        :type test_jnode: dict
        :type step_jnode: dict
        :type args_jnode: None
        """
        if args_jnode:
            eventlog.log_i('fail: http400')
            raise TestArgsError

        result = _get_geodata()
        if result is None:
            return None

        latitude, longitude = result
        step_jnode['data'] = {
            'latitude': {
                'actual': latitude
            },
            'longitude': {
                'actual': longitude
            }
        }

        return step_jnode
