from .test import Test, TestStep, Summary, TestArgsError, TestError

from edge import eventlog
from edge.webserver import cgi
from edge.webserver.sysconf import ph
from edge.webserver.systest import misc
from edge.webserver.sysconf.api_terminal import TerminalApi


def _get_position():
    """
    :rtype: (str, str) | None
    """
    from random import randint
    forward = randint(50, 80)
    left = randint(-2, 4)
    return left, forward


class PositionTestStep(TestStep):
    def __init__(self, services, session_ref, test_ref, step_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        :type step_ref: str
        """
        TestStep.__init__(self, services, session_ref, test_ref, step_ref)

    def _edit_log_item_jnode(self, log_item_jnode, body_jnode):
        """
        :type log_item_jnode: dict
        :type body_jnode: dict
        :rtype: dict
        """
        # unpack #/data/report
        report = body_jnode.get('report', False)
        if type(report) != bool:
            eventlog.log_i('fail: http400. param #/report has invalid value')
            raise cgi.CgiError400

        try:
            data_jnode = body_jnode['data']
            forward_jnode = data_jnode['forward']
            left_jnode = data_jnode['left']

            # unpack #/data/forward/actual
            if forward_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/forward/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/forward/expected
            expected_forward = forward_jnode.get('expected')
            if expected_forward is not None:
                if type(expected_forward) not in [float, int]:
                    eventlog.log_i('fail: http400. param #/data/forward/expected has invalid value')
                    raise cgi.CgiError400

            # unpack #/data/left/actual
            if left_jnode.get('actual'):
                eventlog.log_i('fail: http400. param #/data/left/actual is provided')
                raise cgi.CgiError400

            # unpack #/data/left/expected
            expected_left = left_jnode.get('expected')
            if expected_left is not None:
                if type(expected_left) not in [float, int]:
                    eventlog.log_i('fail: http400. param #/data/left/expected has invalid value')
                    raise cgi.CgiError400

            # unpack #/summary
            summary_jnode = Summary.repack(body_jnode.get('summary'))

            # apply changes
            log_item_jnode['report'] = report
            log_item_jnode['data']['forward']['expected'] = expected_forward
            log_item_jnode['data']['left']['expected'] = expected_left
            log_item_jnode['summary'] = summary_jnode
            return log_item_jnode

        except (KeyError, ValueError) as x:
            eventlog.log_i('fail: http400 <{}>'.format(x.message))
            raise cgi.CgiError400


class PositionTest(Test):
    def __init__(self, services, session_ref, test_ref):
        """
        :type services: Services
        :type session_ref: str
        :type test_ref: (str, str)
        """
        Test.__init__(self, services, session_ref, test_ref)

    def _mk_step_jnode(self, test_jnode, step_jnode, args_jnode):
        """
        :type test_jnode: dict
        :type step_jnode: dict
        :type args_jnode: dict | None

        :raise TestArgsError:
        :raise TestError:
        """
        if args_jnode is None:
            raise TestArgsError

        from edge.com import net
        term_uuid = test_jnode['ro']['origin']
        term_pid = ph.RootInfoUiconf.get_term_pid(term_uuid)

        ra_type = TerminalApi.get_ra_type(term_pid)
        if ra_type is None:
            eventlog.log_e('fail: ra-type is mising')
            raise TestError

        rn_bbox4 = args_jnode.get('bbox-4')
        vs_bbox4 = misc.mk_vs_bbox4(term_pid, rn_bbox4)
        tout = args_jnode.get('tout')
        sym_type = args_jnode.get('simulator')

        try:
            net.query_es_test(self._session_ref, self._test_ref, term_uuid, vs_bbox4, tout)
        except net.NetError:
            eventlog.log_e('fail: setup eventstore')
            raise TestError

        # recongig radar
        speed_filter_args = args_jnode.get('speed')
        ra_filter_speed = 0
        if speed_filter_args:
            speed_kmh, direction = speed_filter_args
            if ra_type == 'r3d':
                ra_filter_speed = speed_kmh
            else:
                speed_mps = speed_kmh / 3.6
                speed_mps = int(round(speed_mps))
                ra_filter_speed = speed_mps if direction == 'receding' else 100 + speed_mps

        try:
            ra_origin_pid = ph.RootInfoUiconf.get_raterm_pid()
            if ra_origin_pid:
                if tout > 0:
                    min_rcs = misc.get_rcs(term_pid, sym_type, systest_active=True)
                    if min_rcs is not None:
                        net.query_ra_alignment(ra_origin_pid, min_rcs=min_rcs, alignment=True, speed_filter=ra_filter_speed)
                else:
                    min_rcs = misc.get_rcs(term_pid, sym_type, systest_active=True)
                    if min_rcs is not None:
                        net.query_ra_alignment(ra_origin_pid, min_rcs=min_rcs, alignment=True)

        except net.NetError:
            eventlog.log_e('fail: setup radar')
            raise TestError
